# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
import logging

from PySide6.QtCore import QCoreApplication, QDir, QFile, QStandardPaths
from PySide6.QtGui import QGuiApplication
from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtSql import QSqlDatabase

# We import the file just to trigger the QmlElement type registration.
import sqlDialog  # noqa E703

logging.basicConfig(filename="chat.log", level=logging.DEBUG)
logger = logging.getLogger("logger")


def connectToDatabase():
    database = QSqlDatabase.database()
    if not database.isValid():
        database = QSqlDatabase.addDatabase("QSQLITE")
        if not database.isValid():
            logger.error("Cannot add database")

    app_data = QStandardPaths.writableLocation(QStandardPaths.StandardLocation.AppDataLocation)
    write_dir = QDir(app_data)
    if not write_dir.mkpath("."):
        logger.error(f"Failed to create writable directory {app_data}")

    # Ensure that we have a writable location on all devices.
    abs_path = write_dir.absolutePath()
    filename = f"{abs_path}/chat-database.sqlite3"

    # When using the SQLite driver, open() will create the SQLite
    # database if it doesn't exist.
    database.setDatabaseName(filename)
    if not database.open():
        logger.error("Cannot open database")
        QFile.remove(filename)


if __name__ == "__main__":
    app = QGuiApplication()
    QCoreApplication.setOrganizationName("QtProject")
    QCoreApplication.setApplicationName("Chat Tutorial")

    connectToDatabase()

    engine = QQmlApplicationEngine()
    engine.addImportPath(sys.path[0])
    engine.loadFromModule("App", "Main")

    if not engine.rootObjects():
        sys.exit(-1)

    exit_code = app.exec()
    del engine
    sys.exit(exit_code)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for inplicit converting C++ numeric types.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
import sys
import sample

# Hardcode the limits of the underlying C-types depending on architecture and memory
# model (taking MSVC using LLP64 into account).
cIntMin = -2147483648
cIntMax = 2147483647
cLongMin = cIntMin
cLongMax = cIntMax
maxRepresentableInt = sys.maxsize
is64bitArchitecture = maxRepresentableInt > 2**32
if is64bitArchitecture and sys.platform != 'win32':
    cLongMin = -9223372036854775808
    cLongMax = 9223372036854775807


class NumericTester(unittest.TestCase):
    '''Helper class for numeric comparison testing'''

    def assertRaises(self, *args, **kwds):
        if not hasattr(sys, "pypy_version_info"):
            # PYSIDE-535: PyPy complains "Fatal RPython error: NotImplementedError"
            return super().assertRaises(*args, **kwds)

    def check_value(self, source, expected, callback, desired_type=None):
        result = callback(source)
        self.assertEqual(result, expected)

        if desired_type:
            self.assertEqual(type(result), desired_type)


class FloatImplicitConvert(NumericTester):
    '''Test case for implicit converting C++ numeric types.'''

    def testFloatAsInt(self):
        '''Float as Int'''
        self.check_value(3.14, 3, sample.acceptInt, int)
        self.assertRaises(OverflowError, sample.acceptInt, cIntMax + 400)

    def testFloatAsLong(self):
        '''Float as Long'''
        #C++ longs are python ints for us
        self.check_value(3.14, 3, sample.acceptLong, int)
        self.assertRaises(OverflowError, sample.acceptLong, cLongMax + 400)

    def testFloatAsUInt(self):
        '''Float as unsigned Int'''
        self.check_value(3.14, 3, sample.acceptUInt, int)
        self.assertRaises(OverflowError, sample.acceptUInt, -3.14)

    def testFloatAsULong(self):
        '''Float as unsigned Long'''
        #FIXME Breaking with SystemError "bad argument to internal function"
        self.check_value(3.14, 3, sample.acceptULong, int)
        self.assertRaises(OverflowError, sample.acceptULong, -3.14)

    def testFloatAsDouble(self):
        '''Float as double'''
        self.check_value(3.14, 3.14, sample.acceptDouble, float)


class IntImplicitConvert(NumericTester):
    '''Test case for implicit converting C++ numeric types.'''

    def testIntAsInt(self):
        '''Int as Int'''
        self.check_value(3, 3, sample.acceptInt, int)

    def testIntAsLong(self):
        '''Int as Long'''
        self.check_value(3, 3, sample.acceptLong, int)

        # cLongMax goes here as CPython implements int as a C long
        self.check_value(cLongMax, cLongMax, sample.acceptLong, int)
        self.check_value(cLongMin, cLongMin, sample.acceptLong, int)

    def testIntAsUInt(self):
        '''Int as unsigned Int'''
        self.check_value(3, 3, sample.acceptUInt, int)
        self.assertRaises(OverflowError, sample.acceptUInt, -3)

    def testIntAsULong(self):
        '''Int as unsigned Long'''
        self.check_value(3, 3, sample.acceptULong, int)
        self.assertRaises(OverflowError, sample.acceptULong, -3)

    def testFloatAsDouble(self):
        '''Float as double'''
        self.check_value(3.14, 3.14, sample.acceptDouble, float)


class LongImplicitConvert(NumericTester):
    '''Test case for implicit converting C++ numeric types.'''

    def testLongAsInt(self):
        '''Long as Int'''
        self.check_value(24224, 24224, sample.acceptInt, int)
        self.assertRaises(OverflowError, sample.acceptInt, cIntMax + 20)

    def testLongAsLong(self):
        '''Long as Long'''
        self.check_value(2405, 2405, sample.acceptLong, int)
        self.assertRaises(OverflowError, sample.acceptLong, cLongMax + 20)

    def testLongAsUInt(self):
        '''Long as unsigned Int'''
        self.check_value(260, 260, sample.acceptUInt, int)
        self.assertRaises(OverflowError, sample.acceptUInt, -42)

    def testLongAsULong(self):
        '''Long as unsigned Long'''
        self.check_value(128, 128, sample.acceptULong, int)
        self.assertRaises(OverflowError, sample.acceptULong, -334)

    def testLongAsDouble(self):
        '''Float as double'''
        self.check_value(42, 42, sample.acceptDouble, float)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QMimeData, Qt, Slot, Signal
from PySide6.QtGui import QPalette, QPixmap
from PySide6.QtWidgets import QFrame, QLabel


class DropArea(QLabel):

    changed = Signal(QMimeData)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setMinimumSize(200, 200)
        self.setFrameStyle(QFrame.Shadow.Sunken | QFrame.Shape.StyledPanel)
        self.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.setAcceptDrops(True)
        self.setAutoFillBackground(True)
        self.clear()

    def dragEnterEvent(self, event):
        self.setText("<drop content>")
        self.setBackgroundRole(QPalette.ColorRole.Highlight)

        event.acceptProposedAction()
        self.changed.emit(event.mimeData())

    def dragMoveEvent(self, event):
        event.acceptProposedAction()

    def dropEvent(self, event):
        mime_data = event.mimeData()

        if mime_data.hasImage():
            self.setPixmap(QPixmap(mime_data.imageData()))
        elif mime_data.hasFormat("text/markdown"):
            self.setText(mime_data.data("text/markdown"))
            self.setTextFormat(Qt.TextFormat.MarkdownText)
        elif mime_data.hasHtml():
            self.setText(mime_data.html())
            self.setTextFormat(Qt.TextFormat.RichText)
        elif mime_data.hasText():
            self.setText(mime_data.text())
            self.setTextFormat(Qt.TextFormat.PlainText)
        elif mime_data.hasUrls():
            url_list = mime_data.urls()
            text = ""
            for i in range(0, min(len(url_list), 32)):
                text += url_list[i].path() + "\n"
            self.setText(text)
        else:
            self.setText("Cannot display data")

        self.setBackgroundRole(QPalette.ColorRole.Dark)
        event.acceptProposedAction()

    def dragLeaveEvent(self, event):
        self.clear()
        event.accept()

    @Slot()
    def clear(self):
        self.setText("<drop content>")
        self.setBackgroundRole(QPalette.ColorRole.Dark)

        self.changed.emit(None)



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from PySide6.QtQuick3D import QQuick3DTextureData
from PySide6.QtQml import QmlElement
from PySide6.QtGui import QImage, QVector3D
from PySide6.QtCore import QByteArray, QObject, Property, Slot, Signal

from request import OSMTileData, OSMRequest

# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "OSMBuildings"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class OSMManager(QObject):

    mapsDataReady = Signal(QByteArray, int, int, int)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_request = OSMRequest(self)
        self.m_startBuildingTileX = 17605
        self.m_startBuildingTileY = 10746
        self.m_tileSizeX = 37
        self.m_tileSizeY = 37
        self.m_request.mapsDataReady.connect(self._slotMapsDataReady)

    def tileSizeX(self):
        return self.m_tileSizeX

    def tileSizeY(self):
        return self.m_tileSizeY

    @Slot(QByteArray, int, int, int)
    def _slotMapsDataReady(self, mapData, tileX, tileY, zoomLevel):
        self.mapsDataReady.emit(mapData, tileX - self.m_startBuildingTileX,
                                tileY - self.m_startBuildingTileY, zoomLevel)

    @Slot(QVector3D, QVector3D, float, float, float, float, float, float)
    def setCameraProperties(self, position, right,
                            cameraZoom, minimumZoom, maximumZoom,
                            cameraTilt, minimumTilt, maximumTilt):

        tiltFactor = (cameraTilt - minimumTilt) / max(maximumTilt - minimumTilt, 1.0)
        zoomFactor = (cameraZoom - minimumZoom) / max(maximumZoom - minimumZoom, 1.0)

        # Forward vector align to the XY plane
        forwardVector = QVector3D.crossProduct(right, QVector3D(0.0, 0.0, -1.0)).normalized()
        projectionOfForwardOnXY = position + forwardVector * tiltFactor * zoomFactor * 50.0

        queue = []
        for forwardIndex in range(-20, 21):
            for sidewardIndex in range(-20, 21):
                vx = float(self.m_tileSizeX * sidewardIndex)
                vy = float(self.m_tileSizeY * forwardIndex)
                transferredPosition = projectionOfForwardOnXY + QVector3D(vx, vy, 0)
                tile_x = self.m_startBuildingTileX + int(transferredPosition.x() / self.m_tileSizeX)
                tile_y = self.m_startBuildingTileY - int(transferredPosition.y() / self.m_tileSizeY)
                self.addBuildingRequestToQueue(queue, tile_x, tile_y)

        projectedTileX = (self.m_startBuildingTileX + int(projectionOfForwardOnXY.x()
                          / self.m_tileSizeX))
        projectedTileY = (self.m_startBuildingTileY - int(projectionOfForwardOnXY.y()
                          / self.m_tileSizeY))

        def tile_sort_key(tile_data):
            return tile_data.distanceTo(projectedTileX, projectedTileY)

        queue.sort(key=tile_sort_key)

        self.m_request.getMapsData(queue.copy())

    def addBuildingRequestToQueue(self, queue, tileX, tileY, zoomLevel=15):
        queue.append(OSMTileData(tileX, tileY, zoomLevel))

    @Slot(result=bool)
    def isDemoToken(self):
        return self.m_request.isDemoToken()

    @Slot(str)
    def setToken(self, token):
        self.m_request.setToken(token)

    @Slot(result=str)
    def token(self):
        return self.m_request.token()

    tileSizeX = Property(int, tileSizeX, constant=True)
    tileSizeY = Property(int, tileSizeY, constant=True)


@QmlElement
class CustomTextureData(QQuick3DTextureData):

    @Slot(QByteArray)
    def setImageData(self, data):
        image = QImage.fromData(data).convertToFormat(QImage.Format.Format_RGBA8888)
        self.setTextureData(QByteArray(bytearray(image.constBits())))
        self.setSize(image.size())
        self.setHasTransparency(False)
        self.setFormat(QQuick3DTextureData.Format.RGBA8)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the widgets/dialogs/tabdialog example from Qt v6.x"""

import sys

from PySide6.QtCore import QFileInfo
from PySide6.QtWidgets import (
    QWidget,
    QVBoxLayout,
    QLabel,
    QCheckBox,
    QApplication,
    QDialog,
    QTabWidget,
    QLineEdit,
    QDialogButtonBox,
    QFrame,
    QListWidget,
    QGroupBox,
)


class TabDialog(QDialog):
    def __init__(self, file_name: str, parent: QWidget = None):
        super().__init__(parent)

        file_info = QFileInfo(file_name)

        tab_widget = QTabWidget()
        tab_widget.addTab(GeneralTab(file_info, self), "General")
        tab_widget.addTab(PermissionsTab(file_info, self), "Permissions")
        tab_widget.addTab(ApplicationsTab(file_info, self), "Applications")

        button_box = QDialogButtonBox(
            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
        )

        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)

        main_layout = QVBoxLayout()
        main_layout.addWidget(tab_widget)
        main_layout.addWidget(button_box)
        self.setLayout(main_layout)
        self.setWindowTitle("Tab Dialog")


class GeneralTab(QWidget):
    def __init__(self, file_info: QFileInfo, parent: QWidget):
        super().__init__(parent)

        file_name_label = QLabel("File Name:")
        file_name_edit = QLineEdit(file_info.fileName())

        path_label = QLabel("Path:")
        path_value_label = QLabel(file_info.absoluteFilePath())
        path_value_label.setFrameStyle(QFrame.Shape.Panel | QFrame.Shadow.Sunken)

        size_label = QLabel("Size:")
        size = file_info.size() / 1024
        size_value_label = QLabel(f"{size} K")
        size_value_label.setFrameStyle(QFrame.Shape.Panel | QFrame.Shadow.Sunken)

        last_read_label = QLabel("Last Read:")
        last_read_value_label = QLabel(file_info.lastRead().toString())
        last_read_value_label.setFrameStyle(QFrame.Shape.Panel | QFrame.Shadow.Sunken)

        last_mod_label = QLabel("Last Modified:")
        last_mod_value_label = QLabel(file_info.lastModified().toString())
        last_mod_value_label.setFrameStyle(QFrame.Shape.Panel | QFrame.Shadow.Sunken)

        main_layout = QVBoxLayout()
        main_layout.addWidget(file_name_label)
        main_layout.addWidget(file_name_edit)
        main_layout.addWidget(path_label)
        main_layout.addWidget(path_value_label)
        main_layout.addWidget(size_label)
        main_layout.addWidget(size_value_label)
        main_layout.addWidget(last_read_label)
        main_layout.addWidget(last_read_value_label)
        main_layout.addWidget(last_mod_label)
        main_layout.addWidget(last_mod_value_label)
        main_layout.addStretch(1)
        self.setLayout(main_layout)


class PermissionsTab(QWidget):
    def __init__(self, file_info: QFileInfo, parent: QWidget):
        super().__init__(parent)

        permissions_group = QGroupBox("Permissions")

        readable = QCheckBox("Readable")
        if file_info.isReadable():
            readable.setChecked(True)

        writable = QCheckBox("Writable")
        if file_info.isWritable():
            writable.setChecked(True)

        executable = QCheckBox("Executable")
        if file_info.isExecutable():
            executable.setChecked(True)

        owner_group = QGroupBox("Ownership")

        owner_label = QLabel("Owner")
        owner_value_label = QLabel(file_info.owner())
        owner_value_label.setFrameStyle(QFrame.Shape.Panel | QFrame.Shadow.Sunken)

        group_label = QLabel("Group")
        group_value_label = QLabel(file_info.group())
        group_value_label.setFrameStyle(QFrame.Shape.Panel | QFrame.Shadow.Sunken)

        permissions_layout = QVBoxLayout()
        permissions_layout.addWidget(readable)
        permissions_layout.addWidget(writable)
        permissions_layout.addWidget(executable)
        permissions_group.setLayout(permissions_layout)

        owner_layout = QVBoxLayout()
        owner_layout.addWidget(owner_label)
        owner_layout.addWidget(owner_value_label)
        owner_layout.addWidget(group_label)
        owner_layout.addWidget(group_value_label)
        owner_group.setLayout(owner_layout)

        main_layout = QVBoxLayout()
        main_layout.addWidget(permissions_group)
        main_layout.addWidget(owner_group)
        main_layout.addStretch(1)
        self.setLayout(main_layout)


class ApplicationsTab(QWidget):
    def __init__(self, file_info: QFileInfo, parent: QWidget):
        super().__init__(parent)

        top_label = QLabel("Open with:")

        applications_list_box = QListWidget()
        applications = []

        for i in range(1, 31):
            applications.append(f"Application {i}")
        applications_list_box.insertItems(0, applications)

        if not file_info.suffix():
            always_check_box = QCheckBox(
                "Always use this application to open this type of file"
            )
        else:
            always_check_box = QCheckBox(
                f"Always use this application to open files "
                f"with the extension {file_info.suffix()}"
            )

        layout = QVBoxLayout()
        layout.addWidget(top_label)
        layout.addWidget(applications_list_box)
        layout.addWidget(always_check_box)
        self.setLayout(layout)


if __name__ == "__main__":
    app = QApplication(sys.argv)

    if len(sys.argv) >= 2:
        file_name = sys.argv[1]
    else:
        file_name = "."

    tab_dialog = TabDialog(file_name)
    tab_dialog.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import sys
import os
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtWidgets import (QApplication, QCommonStyle, QPushButton)


text = ''


class Style(QCommonStyle):

    def drawControl(self, element, option, painter, widget=None):
        # This should be a QStyleOptionButton with a "text" field
        global text
        text = option.text


class StyleOptionTest(UsesQApplication):
    '''PYSIDE-1909: Test cast to derived style option classes.'''

    def testStyle(self):
        global text
        button = QPushButton("Hello World")
        button.setStyle(Style())
        button.show()
        while not text:
            QApplication.processEvents()
        self.assertEqual(text, button.text())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtNetwork import QHostAddress, QTcpServer, QTcpSocket
from helper.usesqapplication import UsesQApplication


class HttpSignalsCase(UsesQApplication):
    '''Test case for launching QHttp signals'''
    DATA = bytes("PySide rocks", "UTF-8")

    def onError(self):
        self.assertTrue(False)

    def onNewConnection(self):
        self.serverConnection = self.server.nextPendingConnection()
        self.serverConnection.errorOccurred.connect(self.onError)
        self.serverConnection.write(HttpSignalsCase.DATA)
        self.server.close()

    def onReadReady(self):
        data = self.client.read(100)
        self.assertEqual(len(data), len(HttpSignalsCase.DATA))
        self.assertEqual(data, HttpSignalsCase.DATA)
        self.done()

    def onClientConnect(self):
        self.client.readyRead.connect(self.onReadReady)

    def initServer(self):
        self.server = QTcpServer()
        self.server.newConnection.connect(self.onNewConnection)
        self.assertTrue(self.server.listen())
        self.client = QTcpSocket()
        self.client.connected.connect(self.onClientConnect)
        self.client.connectToHost(QHostAddress(QHostAddress.SpecialAddress.LocalHost),
                                  self.server.serverPort())

    def done(self):
        self.serverConnection.close()
        self.client.close()
        self.app.quit()

    def testRun(self):
        self.initServer()
        self.app.exec()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Qt, Signal, Slot
from PySide6.QtGui import QPalette
from PySide6.QtWidgets import QPlainTextEdit


UNHANDLED_KEYS = [Qt.Key.Key_Backspace, Qt.Key.Key_Left, Qt.Key.Key_Right, Qt.Key.Key_Up,
                  Qt.Key.Key_Down]


class Console(QPlainTextEdit):

    get_data = Signal(bytearray)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_localEchoEnabled = False
        self.document().setMaximumBlockCount(100)
        p = self.palette()
        p.setColor(QPalette.ColorRole.Base, Qt.GlobalColor.black)
        p.setColor(QPalette.ColorRole.Text, Qt.GlobalColor.green)
        self.setPalette(p)

    @Slot(bytearray)
    def put_data(self, data):
        self.insertPlainText(data.decode("utf8"))
        bar = self.verticalScrollBar()
        bar.setValue(bar.maximum())

    def set_local_echo_enabled(self, e):
        self.m_localEchoEnabled = e

    def keyPressEvent(self, e):
        key = e.key()
        if key not in UNHANDLED_KEYS:
            if self.m_localEchoEnabled:
                super().keyPressEvent(e)
            self.get_data.emit(e.text().encode())

    def mousePressEvent(self, e):
        self.setFocus()

    def mouseDoubleClickEvent(self, e):
        pass

    def contextMenuEvent(self, e):
        pass



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Tests for object reparenting.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
import sys

from sample import ObjectType


class ExtObjectType(ObjectType):
    def __init__(self):
        ObjectType.__init__(self)


class ReparentingTest(unittest.TestCase):
    '''Tests for object reparenting.'''

    def testReparentedObjectTypeIdentity(self):
        '''Reparent children from one parent to another.'''
        object_list = []
        old_parent = ObjectType()
        new_parent = ObjectType()
        for i in range(3):
            obj = ObjectType()
            object_list.append(obj)
            obj.setParent(old_parent)
        for obj in object_list:
            obj.setParent(new_parent)
        for child in new_parent.children():
            self.assertTrue(child in object_list)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testReparentWithTheSameParent(self):
        '''Set the same parent twice to check if the ref continue the same'''
        obj = ObjectType()
        parent = ObjectType()
        base_ref_count = sys.getrefcount(obj)
        obj.setParent(parent)
        self.assertEqual(sys.getrefcount(obj), base_ref_count + 1)
        obj.setParent(parent)
        self.assertEqual(sys.getrefcount(obj), base_ref_count + 1)

    def testReparentedExtObjectType(self):
        '''Reparent children from one extended parent to another.'''
        object_list = []
        old_parent = ExtObjectType()
        new_parent = ExtObjectType()
        for i in range(3):
            obj = ExtObjectType()
            object_list.append(obj)
            obj.setParent(old_parent)
        for obj in object_list:
            obj.setParent(new_parent)
        for orig, child in zip(object_list, new_parent.children()):
            self.assertEqual(type(orig), type(child))

    def testReparentedObjectTypeIdentityWithParentsCreatedInCpp(self):
        '''Reparent children from one parent to another, both created in C++.'''
        object_list = []
        old_parent = ObjectType.create()
        new_parent = ObjectType.create()
        for i in range(3):
            obj = ObjectType()
            object_list.append(obj)
            obj.setParent(old_parent)
        for obj in object_list:
            obj.setParent(new_parent)
        for child in new_parent.children():
            self.assertTrue(child in object_list)

    def testReparentedObjectTypeIdentityWithChildrenCreatedInCpp(self):
        '''Reparent children created in C++ from one parent to another.'''
        object_list = []
        old_parent = ObjectType()
        new_parent = ObjectType()
        for i in range(3):
            obj = ObjectType.create()
            object_list.append(obj)
            obj.setParent(old_parent)
        for obj in object_list:
            obj.setParent(new_parent)
        for child in new_parent.children():
            self.assertTrue(child in object_list)

    def testReparentedObjectTypeIdentityWithParentsAndChildrenCreatedInCpp(self):
        '''Reparent children from one parent to another. Parents and children are created in C++.'''
        object_list = []
        old_parent = ObjectType.create()
        new_parent = ObjectType.create()
        for i in range(3):
            obj = ObjectType.create()
            object_list.append(obj)
            obj.setParent(old_parent)
        for obj in object_list:
            obj.setParent(new_parent)
        for child in new_parent.children():
            self.assertTrue(child in object_list)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import re
from PySide6.QtCore import QMimeData, Qt, Slot
from PySide6.QtGui import QGuiApplication
from PySide6.QtWidgets import (QAbstractItemView, QPushButton,
                               QDialogButtonBox, QLabel,
                               QTableWidget, QTableWidgetItem,
                               QVBoxLayout, QWidget)

from droparea import DropArea

DESCRIPTION = """This example accepts drags from other applications and
displays the MIME types provided by the drag object."""


_WHITESPACE_PATTERN = re.compile(r"\s+")


def simplify_whitespace(s):
    return _WHITESPACE_PATTERN.sub(" ", s).strip()


class DropSiteWindow(QWidget):

    def __init__(self):
        super().__init__()
        drop_area = DropArea()
        abstract_label = QLabel()
        self._formats_table = QTableWidget()

        button_box = QDialogButtonBox()
        abstract_label = QLabel(DESCRIPTION)
        abstract_label.setWordWrap(True)
        abstract_label.adjustSize()

        drop_area = DropArea()
        drop_area.changed.connect(self.update_formats_table)

        self._formats_table = QTableWidget()
        self._formats_table.setColumnCount(2)
        self._formats_table.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self._formats_table.setHorizontalHeaderLabels(["Format", "Content"])
        self._formats_table.horizontalHeader().setStretchLastSection(True)

        clear_button = QPushButton("Clear")
        self._copy_button = QPushButton("Copy")
        quit_button = QPushButton("Quit")

        button_box = QDialogButtonBox()
        button_box.addButton(clear_button, QDialogButtonBox.ButtonRole.ActionRole)
        button_box.addButton(self._copy_button, QDialogButtonBox.ButtonRole.ActionRole)
        self._copy_button.setVisible(False)

        button_box.addButton(quit_button, QDialogButtonBox.ButtonRole.RejectRole)

        quit_button.clicked.connect(self.close)
        clear_button.clicked.connect(drop_area.clear)
        self._copy_button.clicked.connect(self.copy)

        main_layout = QVBoxLayout(self)
        main_layout.addWidget(abstract_label)
        main_layout.addWidget(drop_area)
        main_layout.addWidget(self._formats_table)
        main_layout.addWidget(button_box)

        self.setWindowTitle("Drop Site")
        self.resize(700, 500)

    @Slot(QMimeData)
    def update_formats_table(self, mime_data):
        self._formats_table.setRowCount(0)
        self._copy_button.setEnabled(False)
        if not mime_data:
            return

        for format in mime_data.formats():
            format_item = QTableWidgetItem(format)
            format_item.setFlags(Qt.ItemFlag.ItemIsEnabled)
            format_item.setTextAlignment(Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignLeft)

            if format == "text/plain":
                text = simplify_whitespace(mime_data.text())
            elif format == "text/markdown":
                text = mime_data.data("text/markdown").data().decode("utf8")
            elif format == "text/html":
                text = simplify_whitespace(mime_data.html())
            elif format == "text/uri-list":
                url_list = mime_data.urls()
                text = ""
                for i in range(0, min(len(url_list), 32)):
                    text += url_list[i].toString() + " "
            else:
                data = mime_data.data(format)
                if data.size() > 32:
                    data.truncate(32)
                text = data.toHex(" ").data().decode("utf8").upper()

            row = self._formats_table.rowCount()
            self._formats_table.insertRow(row)
            self._formats_table.setItem(row, 0, QTableWidgetItem(format))
            self._formats_table.setItem(row, 1, QTableWidgetItem(text))

        self._formats_table.resizeColumnToContents(0)
        self._copy_button.setEnabled(self._formats_table.rowCount() > 0)

    @Slot()
    def copy(self):
        text = ""
        for row in range(0, self._formats_table.rowCount()):
            c1 = self._formats_table.item(row, 0).text()
            c2 = self._formats_table.item(row, 1).text()
            text += f"{c1}: {c2}\n"
        QGuiApplication.clipboard().setText(text)



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""
Time a repeated Python run
--------------------------

Usage: python3 lazytiming.py              # uses PySide6
       python3 lazytiming.py <any arg>    # uses PyQt6

It runs the same python for the testing.

Actually comparing PySide6 and PyQt6 in action:

    PYSIDE6_OPTION_LAZY=0 python3 sources/pyside6/tests/manually/lazytiming.py      # normal
    PYSIDE6_OPTION_LAZY=1 python3 sources/pyside6/tests/manually/lazytiming.py      # faster
                          python3 sources/pyside6/tests/manually/lazytiming.py xxx  # PyQt
"""
import subprocess
import sys

from timeit import default_timer as timer

repeats = 100
test1 = "PySide6"
test2 = "PyQt6"

test = test2 if sys.argv[1:] else test1
cmd = [sys.executable, "-c", f"from {test} import QtCore, QtGui, QtWidgets"]

print(f"{repeats} * {test}")

subprocess.call(cmd)        # warmup
start_time = timer()
for idx in range(repeats):
    subprocess.call(cmd)
stop_time = timer()
print(f"time per run = {(stop_time - start_time) / repeats}")



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test bug 635: http://bugs.openbossa.org/show_bug.cgi?id=635'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QApplication, QToolBar


class testQToolBar(unittest.TestCase):
    def callback(self):
        self._called = True

    def testAddAction(self):
        bar = QToolBar()
        self._called = False
        a = bar.addAction("act1", self.callback)
        a.trigger()
        self.assertTrue(self._called)

    def testAddActionWithIcon(self):
        bar = QToolBar()
        self._called = False
        icon = QIcon()
        a = bar.addAction(icon, "act1", self.callback)
        a.trigger()
        self.assertTrue(self._called)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Tests various usages of the type converters.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

import sample


class GetPythonTypeByNameTest(unittest.TestCase):

    '''Uses an added function with inject code that uses the libshiboken
    function "Shiboken::Conversions::getPythonTypeObject(typeName)".'''

    def testGetObjectType(self):
        pyType1 = sample.getPythonType('ObjectType')
        self.assertEqual(pyType1, sample.ObjectType)
        pyType2 = sample.getPythonType('ObjectType*')
        self.assertEqual(pyType2, sample.ObjectType)
        self.assertEqual(pyType1, pyType2)

    def testGetValueType(self):
        pyType1 = sample.getPythonType('Point')
        self.assertEqual(pyType1, sample.Point)
        pyType2 = sample.getPythonType('Point*')
        self.assertEqual(pyType2, sample.Point)
        self.assertEqual(pyType1, pyType2)

    def testGetUsersPrimitiveType(self):
        pyType = sample.getPythonType('OddBool')
        self.assertEqual(pyType, bool)

    def testGetUsersPrimitiveTypeWithoutTargetLangApiName(self):
        '''If the primitive type attribute "target-lang-api-name" is not set
        there'll be no Python type associated with the C++ type.'''
        pyType = sample.getPythonType('PStr')
        self.assertEqual(pyType, None)

    def testPrimitiveTypeAndTypedef(self):
        pyType = sample.getPythonType('double')
        self.assertEqual(pyType, float)
        pyTypedef = sample.getPythonType('real')
        self.assertEqual(pyType, pyTypedef)

    def testPairContainerType(self):
        pyType = sample.getPythonType('std::pair<Complex,int>')
        self.assertEqual(pyType, list)

    def testListContainerType(self):
        pyType = sample.getPythonType('std::list<int>')
        self.assertEqual(pyType, list)

    def testMapContainerType(self):
        pyType = sample.getPythonType('std::map<std::string,int>')
        self.assertEqual(pyType, dict)

    def testGlobalEnumType(self):
        pyType = sample.getPythonType('GlobalEnum')
        self.assertEqual(pyType, sample.GlobalEnum)

    def testScopedEnumType(self):
        pyType = sample.getPythonType('Abstract::Type')
        self.assertEqual(pyType, sample.Abstract.Type)


class CheckValueAndObjectTypeByNameTest(unittest.TestCase):

    '''Uses an added function with inject code that uses the libshiboken
    functions that check if a type is Object or Value, based on its converter.'''

    def testErrors(self):
        '''not existent type'''
        self.assertRaises(ValueError, sample.cppTypeIsValueType, 'NotExistentType')
        self.assertRaises(ValueError, sample.cppTypeIsObjectType, 'NotExistentType')

    def testObjectType1(self):
        self.assertTrue(sample.cppTypeIsObjectType('ObjectType'))
        self.assertFalse(sample.cppTypeIsValueType('ObjectType'))

    def testObjectType2(self):
        self.assertTrue(sample.cppTypeIsObjectType('ObjectType*'))
        self.assertFalse(sample.cppTypeIsValueType('ObjectType*'))

    def testValueType1(self):
        self.assertTrue(sample.cppTypeIsValueType('Point'))
        self.assertFalse(sample.cppTypeIsObjectType('Point'))

    def testValueType2(self):
        self.assertTrue(sample.cppTypeIsValueType('Point*'))
        self.assertFalse(sample.cppTypeIsObjectType('Point*'))

    def testUsersPrimitiveType(self):
        self.assertFalse(sample.cppTypeIsValueType('Complex'))
        self.assertFalse(sample.cppTypeIsObjectType('Complex'))

    def testContainerType(self):
        self.assertFalse(sample.cppTypeIsValueType('std::list<int>'))
        self.assertFalse(sample.cppTypeIsObjectType('std::list<int>'))


class SpecificConverterTest(unittest.TestCase):

    '''Uses an added function with inject code that uses the libshiboken
    adapter class "Shiboken::Conversions::SpecificConverter".'''

    def testNotExistentType(self):
        conversion = sample.getConversionTypeString('NotExistentType')
        self.assertEqual(conversion, 'Invalid conversion')

    def testObjectType(self):
        conversion = sample.getConversionTypeString('ObjectType')
        self.assertEqual(conversion, 'Pointer conversion')
        conversion = sample.getConversionTypeString('ObjectType*')
        self.assertEqual(conversion, 'Pointer conversion')
        conversion = sample.getConversionTypeString('ObjectType&')
        self.assertEqual(conversion, 'Reference conversion')

    def testValueType(self):
        conversion = sample.getConversionTypeString('Point')
        self.assertEqual(conversion, 'Copy conversion')
        conversion = sample.getConversionTypeString('Point*')
        self.assertEqual(conversion, 'Pointer conversion')
        conversion = sample.getConversionTypeString('Point&')
        self.assertEqual(conversion, 'Reference conversion')


class StringBasedConversionTest(unittest.TestCase):

    def testValueType(self):
        pts = (sample.Point(1, 1), sample.Point(2, 2), sample.Point(3, 3))
        result = sample.convertValueTypeToCppAndThenToPython(pts[0], pts[1], pts[2])
        for orig, new in zip(pts, result):
            self.assertEqual(orig, new)
        self.assertFalse(pts[0] is result[0])
        self.assertTrue(pts[1] is result[1])
        self.assertTrue(pts[2] is result[2])

    def testObjectType(self):
        objs = (sample.ObjectType(), sample.ObjectType())
        objs[0].setObjectName('obj0')
        objs[1].setObjectName('obj1')
        result = sample.convertObjectTypeToCppAndThenToPython(objs[0], objs[1])
        for orig, new in zip(objs, result):
            self.assertEqual(orig, new)
            self.assertEqual(orig.objectName(), new.objectName())
            self.assertTrue(orig is new)

    def testContainerType(self):
        lst = range(4)
        result = sample.convertListOfIntegersToCppAndThenToPython(lst)
        self.assertTrue(len(result), 1)
        self.assertTrue(lst, result[0])


class PrimitiveConversionTest(unittest.TestCase):

    def testCppPrimitiveType(self):
        integers = (12, 34)
        result = sample.convertIntegersToCppAndThenToPython(integers[0], integers[1])
        for orig, new in zip(integers, result):
            self.assertEqual(orig, new)

    def testLargeIntAsFloat(self):
        """PYSIDE-2417: When passing an int to a function taking float,
           a 64bit conversion should be done."""
        point = sample.PointF(1, 2)
        large_int = 2**31 + 2
        point.setX(large_int)
        self.assertEqual(round(point.x()), large_int)

    def testUnsignedLongLongAsFloat(self):
        """PYSIDE-2652: When passing an unsigned long long to a function taking float,
           an unsigned 64bit conversion should be done."""
        point = sample.PointF(1, 2)
        large_int = 2**63
        point.setX(large_int)
        self.assertEqual(round(point.x()), large_int)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import json
from pathlib import Path
from PySide6.QtCore import Slot, QObject, Property, Signal
from PySide6.QtGui import QColor
from PySide6.QtQml import QmlNamedElement, QmlSingleton

QML_IMPORT_NAME = "FileSystemModule"
QML_IMPORT_MAJOR_VERSION = 1


@QmlNamedElement("Colors")
@QmlSingleton
class SchemeManager(QObject):

    schemeChanged = Signal()

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        with open(Path(__file__).parent / "schemes.json", 'r') as f:
            self.m_schemes = json.load(f)
        self.m_activeScheme = {}
        self.m_activeSchemeName = "Catppuccin"
        self.setScheme(self.m_activeSchemeName)

    @Slot(str)
    def setScheme(self, theme):
        for k, v in self.m_schemes[theme].items():
            self.m_activeScheme[k] = QColor.fromString(v)
        self.m_activeSchemeName = theme
        self.schemeChanged.emit()

    @Slot(result='QStringList')
    def getKeys(self):
        return self.m_schemes.keys()

    @Property('QStringList', notify=schemeChanged)
    def currentColors(self):
        return self.m_schemes[self.m_activeSchemeName].values()

    @Property(QColor, notify=schemeChanged)
    def background(self):
        return self.m_activeScheme["background"]

    @Property(QColor, notify=schemeChanged)
    def surface1(self):
        return self.m_activeScheme["surface1"]

    @Property(QColor, notify=schemeChanged)
    def surface2(self):
        return self.m_activeScheme["surface2"]

    @Property(QColor, notify=schemeChanged)
    def text(self):
        return self.m_activeScheme["text"]

    @Property(QColor, notify=schemeChanged)
    def textFile(self):
        return self.m_activeScheme["textFile"]

    @Property(QColor, notify=schemeChanged)
    def disabledText(self):
        return self.m_activeScheme["disabledText"]

    @Property(QColor, notify=schemeChanged)
    def selection(self):
        return self.m_activeScheme["selection"]

    @Property(QColor, notify=schemeChanged)
    def active(self):
        return self.m_activeScheme["active"]

    @Property(QColor, notify=schemeChanged)
    def inactive(self):
        return self.m_activeScheme["inactive"]

    @Property(QColor, notify=schemeChanged)
    def folder(self):
        return self.m_activeScheme["folder"]

    @Property(QColor, notify=schemeChanged)
    def icon(self):
        return self.m_activeScheme["icon"]

    @Property(QColor, notify=schemeChanged)
    def iconIndicator(self):
        return self.m_activeScheme["iconIndicator"]

    @Property(QColor, notify=schemeChanged)
    def color1(self):
        return self.m_activeScheme["color1"]

    @Property(QColor, notify=schemeChanged)
    def color2(self):
        return self.m_activeScheme["color2"]



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QImage'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QImage, qRgb

from helper.usesqapplication import UsesQApplication


class SetPixelFloat(UsesQApplication):
    '''Test case for calling setPixel with float as argument'''

    def setUp(self):
        # Acquire resources
        super(SetPixelFloat, self).setUp()
        self.color = qRgb(255, 0, 0)
        self.image = QImage(200, 200, QImage.Format.Format_RGB32)

    def tearDown(self):
        # Release resources
        del self.color
        del self.image
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super(SetPixelFloat, self).tearDown()

    def testFloat(self):
        # QImage.setPixel(float, float, color) - Implicit conversion
        self.image.setPixel(3.14, 4.2, self.color)
        self.assertEqual(self.image.pixel(3.14, 4.2), self.color)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 WebEngine QtQuick 2 Example"""

import sys
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path

from PySide6.QtCore import (QCoreApplication, QFileInfo, QMetaObject, QObject,
                            QUrl, Slot, Q_ARG)
from PySide6.QtQml import QQmlApplicationEngine, QmlElement, QmlSingleton
from PySide6.QtGui import QGuiApplication
from PySide6.QtWebEngineQuick import QtWebEngineQuick

import rc_resources  # noqa: F401


# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "BrowserUtils"
QML_IMPORT_MAJOR_VERSION = 1


def url_from_user_input(user_input):
    file_info = QFileInfo(user_input)
    if file_info.exists():
        return QUrl.fromLocalFile(file_info.absoluteFilePath())
    return QUrl.fromUserInput(user_input)


@QmlElement
@QmlSingleton
class Utils(QObject):

    @Slot(str, result=QUrl)
    def fromUserInput(self, user_input):
        return url_from_user_input(user_input)


if __name__ == '__main__':
    QCoreApplication.setApplicationName("Quick Nano Browser")
    QCoreApplication.setOrganizationName("QtProject")

    QtWebEngineQuick.initialize()

    argument_parser = ArgumentParser(description="Quick Nano Browser",
                                     formatter_class=RawTextHelpFormatter)
    argument_parser.add_argument("--single-process", "-s", action="store_true",
                                 help="Run in single process mode (trouble shooting)")
    argument_parser.add_argument("url", help="The URL to open",
                                 nargs='?', type=str)
    options = argument_parser.parse_args()

    url = url_from_user_input(options.url) if options.url else QUrl("chrome://qt")

    app_args = sys.argv
    if options.single_process:
        app_args.extend(["--webEngineArgs", "--single-process"])
    app = QGuiApplication(app_args)
    engine = QQmlApplicationEngine()
    engine.addImportPath(Path(__file__).parent)
    engine.loadFromModule("BrowserUtils", "ApplicationRoot")
    if not engine.rootObjects():
        sys.exit(-1)

    QMetaObject.invokeMethod(engine.rootObjects()[0], "load", Q_ARG("QVariant", url))

    exit_code = app.exec()
    del engine
    sys.exit(exit_code)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QPoint, QPointF
from PySide6.QtCore import QLine, QLineF
from PySide6.QtCore import QSize, QSizeF


class testCases(unittest.TestCase):
    def testQPointToTuple(self):
        p = QPoint(1, 2)
        self.assertEqual((1, 2), p.toTuple())

    def testQPointFToTuple(self):
        p = QPointF(1, 2)
        self.assertEqual((1, 2), p.toTuple())

    def testQLineToTuple(self):
        line = QLine(1, 2, 3, 4)
        self.assertEqual((1, 2, 3, 4), line.toTuple())

    def testQLineFToTuple(self):
        line = QLineF(1, 2, 3, 4)
        self.assertEqual((1, 2, 3, 4), line.toTuple())

    def testQSizeToTuple(self):
        s = QSize(1, 2)
        self.assertEqual((1, 2), s.toTuple())

    def testQSizeFToTuple(self):
        s = QSizeF(1, 2)
        self.assertEqual((1, 2), s.toTuple())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QtSql database creation, destruction and queries'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtSql import QSqlDatabase, QSqlQuery, QSqlTableModel
from PySide6.QtWidgets import QWidget
from helper.usesqapplication import UsesQApplication


class Foo(QWidget):
    def __init__(self):
        super().__init__()
        self.model = QSqlTableModel()


class SqlDatabaseCreationDestructionAndQueries(UsesQApplication):
    '''Test cases for QtSql database creation, destruction and queries'''

    def setUp(self):
        # Acquire resources
        super().setUp()
        self.assertFalse(not QSqlDatabase.drivers(), "installed Qt has no DB drivers")
        self.assertTrue("QSQLITE" in QSqlDatabase.drivers(),
                        "\"QSQLITE\" driver not available in this Qt version")
        self.db = QSqlDatabase.addDatabase("QSQLITE")
        self.db.setDatabaseName(":memory:")
        self.assertTrue(self.db.open())

    def tearDown(self):
        # Release resources
        self.db.close()
        QSqlDatabase.removeDatabase(":memory:")
        del self.db
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super().tearDown()

    def testTableCreationAndDestruction(self):
        # Test table creation and destruction
        query = QSqlQuery()
        query.exec("CREATE TABLE dummy(id int primary key, dummyfield varchar(20))")
        query.exec("DROP TABLE dummy")
        query.clear()

    def testTableInsertionAndRetrieval(self):
        # Test table creation, insertion and retrieval
        query = QSqlQuery()
        query.exec("CREATE TABLE person(id int primary key, "
                   "firstname varchar(20), lastname varchar(20))")
        query.exec("INSERT INTO person VALUES(101, 'George', 'Harrison')")
        query.prepare("INSERT INTO person (id, firstname, lastname) "
                      "VALUES (:id, :firstname, :lastname)")
        query.bindValue(":id", 102)
        query.bindValue(":firstname", "John")
        query.bindValue(":lastname", "Lennon")
        query.exec()

        lastname = ''
        query.exec("SELECT lastname FROM person where id=101")
        self.assertTrue(query.isActive())
        query.next()
        lastname = query.value(0)
        self.assertEqual(lastname, 'Harrison')

    def testTableModelDeletion(self):
        bar = Foo()
        model = bar.model  # noqa: F841
        del bar
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for overloads involving static and non-static versions of a method.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import SimpleFile


class SimpleFile2 (SimpleFile):
    def exists(self):
        return "Mooo"


class SimpleFile3 (SimpleFile):
    pass


class SimpleFile4 (SimpleFile):
    exists = 5


class StaticNonStaticMethodsTest(unittest.TestCase):
    '''Test cases for overloads involving static and non-static versions of a method.'''

    def setUp(self):
        filename = f'simplefile{os.getpid()}.txt'
        self.existing_filename = Path(filename)
        self.delete_file = False
        if not self.existing_filename.exists():
            with self.existing_filename.open('w') as f:
                for line in range(10):
                    f.write('sbrubbles\n')
            self.delete_file = True

        self.non_existing_filename = Path('inexistingfile.txt')
        i = 0
        while self.non_existing_filename.exists():
            i += 1
            filename = 'inexistingfile-{i}.txt'
            self.non_existing_filename = Path(filename)

    def tearDown(self):
        if self.delete_file:
            os.remove(self.existing_filename)

    def testCallingStaticMethodWithClass(self):
        '''Call static method using class.'''
        self.assertTrue(SimpleFile.exists(os.fspath(self.existing_filename)))
        self.assertFalse(SimpleFile.exists(os.fspath(self.non_existing_filename)))

    def testCallingStaticMethodWithInstance(self):
        '''Call static method using instance of class.'''
        f = SimpleFile(os.fspath(self.non_existing_filename))
        self.assertTrue(f.exists(os.fspath(self.existing_filename)))
        self.assertFalse(f.exists(os.fspath(self.non_existing_filename)))

    def testCallingInstanceMethod(self):
        '''Call instance method.'''
        f1 = SimpleFile(os.fspath(self.non_existing_filename))
        self.assertFalse(f1.exists())
        f2 = SimpleFile(os.fspath(self.existing_filename))
        self.assertTrue(f2.exists())

    def testOverridingStaticNonStaticMethod(self):
        f = SimpleFile2(os.fspath(self.existing_filename))
        self.assertEqual(f.exists(), "Mooo")

        f = SimpleFile3(os.fspath(self.existing_filename))
        self.assertTrue(f.exists())

        f = SimpleFile4(os.fspath(self.existing_filename))
        self.assertEqual(f.exists, 5)

    def testDuckPunchingStaticNonStaticMethod(self):
        f = SimpleFile(os.fspath(self.existing_filename))
        f.exists = lambda: "Meee"
        self.assertEqual(f.exists(), "Meee")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

from converter import snippet_translate as st


def multi_st(lines):
    result = [st(line) for line in lines.split("\n")]
    return "\n".join(result)


def test_comments():
    assert st("// This is a comment") == "# This is a comment"
    assert st("// double slash // inside") == "# double slash // inside"


def test_comments_eol():
    assert st("a = 1; // comment") == "a = 1 # comment"
    assert st("while ( 1 != 1 ) { // comment") == "while 1 != 1: # comment"


def test_qdoc_snippets():
    assert st("//! [0]") == "//! [0]"


def test_arrow():
    assert st("label->setText('something')") == "label.setText('something')"


def test_curly_braces():
    assert st("        {") == ""
    assert st("}") == ""
    assert st("while (true){") == "while True:"
    assert st("while (true)   {  ") == "while True:"


def test_inc_dec():
    assert st("++i;") == "i += 1"
    assert st("i--;") == "i -= 1"


def test_and_or():
    assert st("while (a && b)") == "while a and b:"
    assert st("else if (a || b && c)") == "elif a or b and c:"


def test_while_if_elseif():
    assert st("while(a)") == "while a:"
    assert st("if  (condition){") == "if condition:"
    assert st("    if  (condition){") == "    if condition:"
    assert st("} else if (a) {") == " elif a:"
    assert (
        st("if (!m_vbo.isCreated()) // init() failed,")
        == "if not m_vbo.isCreated(): # init() failed,"
    )
    # Special case, second line from a condition
    assert (
        st("&& event->answerRect().intersects(dropFrame->geometry()))")
        == "and event.answerRect().intersects(dropFrame.geometry()))"
    )


def test_else():
    assert st("else") == "else:"
    assert st("} else {") == "else:"
    assert st("}else") == "else:"
    assert st("else {") == "else:"


def test_qualify_enumerations():
    assert st("Qt::Key_A") == "Qt.Key.Key_A"


def test_new():
    assert st("a = new Something(...);") == "a = Something(...)"
    assert st("a = new Something") == "a = Something()"
    assert st("foo(new X, new Y(b), new Z)") == "foo(X(), Y(b), Z())"
    # Class member initialization list
    assert st("m_mem(new Something(p)),") == "m_mem(Something(p)),"
    assert st("m_mem(new Something),") == "m_mem(Something()),"


def test_semicolon():
    assert st("a = 1;") == "a = 1"
    assert st("};") == ""


def test_include():
    assert st('#include "something.h"') == "from something import *"
    assert st("#include <QtCore>") == "from PySide6 import QtCore"
    assert st("#include <QLabel>") == "from PySide6.QtWidgets import QLabel"
    assert st("#include <NotQt>") == ""
    assert st('#include strange"') == ""


def test_main():
    assert st("int main(int argc, char *argv[])") == 'if __name__ == "__main__":'


def test_cast():
    assert st("a = reinterpret_cast<type>(data);") == "a = type(data)"
    assert st("a = reinterpret_cast<type*>(data) * 9;") == "a = type(data) * 9"
    assert (
        st("elapsed = (elapsed + qobject_cast<QTimer*>(sender())->interval()) % 1000;")
        == "elapsed = (elapsed + QTimer(sender()).interval()) % 1000"
    )
    assert (
         st("a = qobject_cast<type*>(data) * 9 + static_cast<int>(42)")  # noqa: E126
         == "a = type(data) * 9 + int(42)"
    )


def test_double_colon():
    assert st("Qt::Align") == "Qt.Align"
    assert st('QSound::play("mysounds/bells.wav");') == 'QSound.play("mysounds/bells.wav")'
    assert st("Widget::method") == "Widget.method"

    # multiline statement connect statement
    # eg: connect(reply, &QNetworkReply::errorOccurred,
    #    this, &MyClass::slotError);
    assert st("this, &MyClass::slotError);") == "self.slotError)"


def test_connects():
    assert (
        st("connect(button, &QPushButton::clicked, this, &MyClass::slotClicked);")
        == "button.clicked.connect(self.slotClicked)"
    )
    assert (
        st("connect(m_ui->button, &QPushButton::clicked, this, &MyClass::slotClicked);")
        == "m_ui.button.clicked.connect(self.slotClicked)"
    )
    assert (
        st("connect(button.get(), &QPushButton::clicked, this, &MyClass::slotClicked);")
        == "button.clicked.connect(self.slotClicked)"
    )


def test_cout_endl():
    assert st("cout << 'hello' << 'world' << endl") == "print('hello', 'world')"
    assert st("    cout << 'hallo' << 'welt' << endl") == "    print('hallo', 'welt')"
    assert st("cout << 'hi'") == "print('hi')"
    assert st("'world' << endl") == "print('world')"

    assert st("cout << circ.at(i) << endl;") == "print(circ.at(i))"
    assert (
        st('cout << "Element name: " << qPrintable(e.tagName()) << "\n";')
        == 'print("Element name: ", qPrintable(e.tagName()), "\n")'
    )
    assert (
        st('cout << "First occurrence of Harumi is at position " << i << Qt::endl;')
        == 'print("First occurrence of Harumi is at position ", i)'
    )
    assert st('cout << "Found Jeanette" << endl;') == 'print("Found Jeanette")'
    assert st('cout << "The key: " << it.key() << Qt::endl') == 'print("The key: ", it.key())'
    assert (
        st("cout << (*constIterator).toLocal8Bit().constData() << Qt::endl;")
        == "print((constIterator).toLocal8Bit().constData())"
    )
    assert st("cout << ba[0]; // prints H") == "print(ba[0]) # prints H"
    assert (
        st('cout << "Also the value: " << (*it) << Qt::endl;') == 'print("Also the value: ", (it))'
    )
    assert st('cout << "[" << *data << "]" << Qt::endl;') == 'print("[", data, "]")'

    assert st('out << "Qt rocks!" << Qt::endl;') == 'print(out, "Qt rocks!")'
    assert st('    std::cout << "MyObject::MyObject()\n";') == '    print("MyObject::MyObject()\n")'
    assert st('qDebug() << "Retrieved:" << retrieved;') == 'print("Retrieved:", retrieved)'


def test_variable_declaration():
    assert st("QLabel label;") == "label = QLabel()"
    assert st('QLabel label("Hello");') == 'label = QLabel("Hello")'
    assert st("Widget w;") == "w = Widget()"
    assert st('QLabel *label = new QLabel("Hello");') == 'label = QLabel("Hello")'
    assert st('QLabel label = a_function("Hello");') == 'label = a_function("Hello")'
    assert st('QString a = "something";') == 'a = "something"'
    assert st("int var;") == "var = int()"
    assert st("float v = 0.1;") == "v = 0.1"
    assert st("QSome<thing> var;") == "var = QSome()"
    assert st("QQueue<int> queue;") == "queue = QQueue()"
    assert st("QVBoxLayout *layout = new QVBoxLayout;") == "layout = QVBoxLayout()"
    assert st("QPointer<QLabel> label = new QLabel;") == "label = QLabel()"
    assert st("QMatrix4x4 matrix;") == "matrix = QMatrix4x4()"
    assert st("QList<QImage> collage =") == "collage ="
    assert st("bool b = true;") == "b = True"
    assert st("Q3DBars *m_graph = nullptr;") == "m_graph = None"
    # Do not fall for member function definitions
    assert st("Q3DBars *Graph::bars() const") == "Q3DBars Graph.bars()"
    # Do not fall for member function declarations
    assert st("virtual Q3DBars *bars();") == "virtual Q3DBars bars()"


def test_for():
    assert st("for (int i = 0; i < 10; i++)") == "for i in range(0, 10):"
    assert st("    for (int i = 0; i < 10; i+=2)") == "    for i in range(0, 10, 2):"
    assert st("for (int i = 10; i >= 0; i-=2)") == "for i in range(-1, 10, -2):"
    assert st("for (int i = 0; i < 10; ++i)") == "for i in range(0, 10):"
    assert (
        st("for (int c = 0;" "c < model.columnCount();" "++c) {")
        == "for c in range(0, model.columnCount()):"
    )
    assert (
        st("for (int c = 0;" "c < table->columns();" "++c) {")
        == "for c in range(0, table.columns()):"
    )
    assert st("for (int i = 0; i <= 10; i++)") == "for i in range(0, 11):"
    assert st("for (int i = 10; i >= 0; i--)") == "for i in range(-1, 10, -1):"

    # # if contains "begin()" and "end()", do a 'for it in var'
    assert (
        st(
            "for (QHash<int, QString>::const_iterator it = hash.cbegin(),"
            "end = hash.cend(); it != end; ++it)"
        )
        == "for it in hash:"
    )
    assert (
        st("for (QTextBlock it = doc->begin();" "it != doc->end(); it = it.next())")
        == "for it in doc:"
    )
    assert st("for (auto it = map.begin(); it != map.end(); ++it) {") == "for it in map:"
    assert st("for (i = future.constBegin(); i != future.constEnd(); ++i)") == "for i in future:"
    assert st("for (it = block.begin(); !(it.atEnd()); ++it) {") == "for it in block:"
    assert (
        st("    for (it = snippetPaths.constBegin();" "it != snippetPaths.constEnd(); ++it)")
        == "    for it in snippetPaths:"
    )

    assert st("for (QChar ch : s)") == "for ch in s:"
    assert (
        st("for (const QByteArray &ext : " "qAsConst(extensionList))")
        == "for ext in extensionList:"
    )
    assert st("for (QTreeWidgetItem *item : found) {") == "for item in found:"

    # TODO: Strange cases
    # for ( ; it != end; ++it) {
    # for (; !elt.isNull(); elt = elt.nextSiblingElement("entry")) {
    # for (int i = 0; ids[i]; ++i)
    # for (int i = 0; i < (1>>20); ++i)
    # for(QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling())


def test_emit():
    assert st("emit sliderPressed();") == "sliderPressed.emit()"
    assert st("emit actionTriggered(action);") == "actionTriggered.emit(action)"
    assert st("emit activeChanged(d->m_active);") == "activeChanged.emit(d.m_active)"
    assert st("emit dataChanged(index, index);") == "dataChanged.emit(index, index)"
    assert st("emit dataChanged(index, index, {role});") == "dataChanged.emit(index, index, {role})"
    assert (
        st('emit dragResult(tr("The data was copied here."));')
        == 'dragResult.emit(tr("The data was copied here."))'
    )
    assert (
        st("emit mimeTypes(event->mimeData()->formats());")
        == "mimeTypes.emit(event.mimeData().formats())"
    )
    assert (
        st("emit q_ptr->averageFrequencyChanged(m_averageFrequency);")
        == "q_ptr.averageFrequencyChanged.emit(m_averageFrequency)"
    )
    assert st("emit q_ptr->frequencyChanged();") == "q_ptr.frequencyChanged.emit()"
    assert (
        st("emit rangeChanged(d->minimum, d->maximum);")
        == "rangeChanged.emit(d.minimum, d.maximum)"
    )
    assert (
        st("emit sliderMoved((d->position = value));") == "sliderMoved.emit((d.position = value))"
    )
    assert (
        st("emit stateChanged(QContactAction::FinishedState);")
        == "stateChanged.emit(QContactAction.FinishedState)"
    )
    assert st("emit textCompleted(lineEdit->text());") == "textCompleted.emit(lineEdit.text())"
    assert (
        st("emit updateProgress(newstat, m_watcher->progressMaximum());")
        == "updateProgress.emit(newstat, m_watcher.progressMaximum())"
    )


def test_void_functions():
    assert st("void Something::Method(int a, char *b) {") == "def Method(self, a, b):"
    assert (
        st("void MainWindow::updateMenus(QListWidgetItem *current)")
        == "def updateMenus(self, current):"
    )
    assert (
        st("void MyScrollArea::scrollContentsBy(int dx, int dy)")
        == "def scrollContentsBy(self, dx, dy):"
    )
    assert st("void Wrapper::wrapper6() {") == "def wrapper6(self):"
    assert st("void MyClass::setPriority(Priority) {}") == "def setPriority(self, Priority): pass"
    assert st("void MyException::raise() const { throw *this; }") == "def raise(self): raise self"
    assert st("void tst_Skip::test_data()") == "def test_data(self):"
    assert st("void util_function_does_nothing()") == "def util_function_does_nothing():"
    assert st("static inline void cleanup(MyCustomClass *pointer)") == "def cleanup(pointer):"
    # TODO: Which name?
    assert st("void RenderWindow::exposeEvent(QExposeEvent *)") == "def exposeEvent(self, arg__0):"


def test_classes():
    assert st("class MyWidget //: public QWidget") == "class MyWidget(): #: public QWidget"
    assert st("class MyMfcView : public CView") == "class MyMfcView(CView):"
    assert st("class MyGame : public QObject {") == "class MyGame(QObject):"
    assert st("class tst_Skip") == "class tst_Skip():"
    assert st("class A : public B, protected C") == "class A(B, C):"
    assert st("class A : public B, public C") == "class A(B, C):"
    assert st("class SomeTemplate<int> : public QFrame") == "class SomeTemplate(QFrame):"
    # This is a tricky situation because it has a multi line dependency:
    #   class MyMemberSheetExtension : public QObject,
    #          public QDesignerMemberSheetExtension
    #   {
    # we will use the leading comma to trust it's the previously situation.
    assert st("class A : public QObject,") == "class A(QObject,"
    assert st("class B {...};") == "class B(): pass"


def test_constuctors():
    assert st("MyWidget::MyWidget(QWidget *parent)") == "def __init__(self, parent):"
    assert st("Window::Window()") == "def __init__(self):"


def test_inheritance_init():
    assert (
        st(": QClass(fun(re, 1, 2), parent), a(1)")
        == "    super().__init__(fun(re, 1, 2), parent)\n    self.a = 1"
    )
    assert (
        st(":   QQmlNdefRecord(copyFooRecord(record), parent)")
        == "    super().__init__(copyFooRecord(record), parent)"
    )
    assert (
        st("    : QWidget(parent), helper(helper)")
        == "    super().__init__(parent)\n    self.helper = helper"
    )
    assert st("   : QWidget(parent)") == "    super().__init__(parent)"
    assert (
        st(": a(0), bB(99), cC2(1), p_S(10),")
        == "    self.a = 0\n    self.bB = 99\n    self.cC2 = 1\n    self.p_S = 10"
    )
    assert (
        st(": QAbstractFileEngineIterator(nameFilters, filters), index(0) ")
        == "    super().__init__(nameFilters, filters)\n    self.index = 0"
    )
    assert (
        st(": m_document(doc), m_text(text)") == "    self.m_document = doc\n    self.m_text = text"
    )
    assert st(": m_size(size) { }") == "    self.m_size = size"
    assert (
        st(": option->palette.color(QPalette::Mid);")
        == "    self.option.palette.color = QPalette.Mid"
    )
    assert st(": QSqlResult(driver) {}") == "    super().__init__(driver)"


def test_arrays():
    assert st("static const GLfloat vertices[] = {") == "vertices = {"
    assert st("static const char *greeting_strings[] = {") == "greeting_strings = {"
    assert st("uchar arrow_bits[] = {0x3f, 0x1f, 0x0f}") == "arrow_bits = {0x3f, 0x1f, 0x0f}"
    assert st("QList<int> vector { 1, 2, 3, 4 };") == "vector = { 1, 2, 3, 4 }"


def test_functions():
    assert st("int Class::method(a, b, c)") == "def method(self, a, b, c):"
    assert st("QStringView Message::body() const") == "def body(self):"
    assert st("void Ren::exEvent(QExp *)") == "def exEvent(self, arg__0):"
    assert (
        st("QString myDecoderFunc(const QByteArray &localFileName);")
        == "def myDecoderFunc(localFileName):"
    )
    assert st("return QModelIndex();") == "return QModelIndex()"


def test_foreach():
    assert st("foreach (item, selected) {") == "for item in selected:"
    assert st("foreach (const QVariant &v, iterable) {") == "for v in iterable:"
    assert st("foreach (QObject *obj, list)") == "for obj in list:"
    assert (
        st("foreach (const QContactTag& tag, contact.details<QContactTag>()) {")
        == "for tag in contact.details():"
    )


def test_structs():
    assert st("struct ScopedPointerCustomDeleter") == "class ScopedPointerCustomDeleter():"
    assert st("struct Wrapper : public QWidget {") == "class Wrapper(QWidget):"
    assert st("struct Window {") == "class Window():"


def test_ternary_operator():
    assert st("int a = 1 ? b > 0 : 3") == "a = 1 if b > 0 else 3"
    assert (
        st("if (!game.saveGame(json ? Game::Json : Game::Binary))")
        == "if not game.saveGame(json if Game.Json else Game.Binary):"
    )


def test_useless_qt_classes():
    assert st('result += QLatin1String("; ");') == 'result += "; "'
    assert st('result += QString::fromLatin1("; ");') == 'result += "; "'
    assert (
        st('result = QStringLiteral("A") + QStringLiteral("B");')
        == 'result = "A" + "B"')
    assert st("<< QLatin1Char('\0') << endl;") == "print('\0')"
    assert st('result = u"A"_s;') == 'result = "A"'


def test_special_cases():
    assert (
        st('http->setProxy("proxy.example.com", 3128);')
        == 'http.setProxy("proxy.example.com", 3128)'
    )
    assert st("delete something;") == "del something"
    assert (
        st("m_program->setUniformValue(m_matrixUniform, matrix);")
        == "m_program.setUniformValue(m_matrixUniform, matrix)"
    )
    assert (
        st("QObject::connect(&window1, &Window::messageSent,")
        == "window1.messageSent.connect("
    )
    assert st("double num;") == "num = float()"

    # Leave a comment to remember it comes from C++
    assert st("public:") == "# public"
    assert st("private:") == "# private"

    #iterator declaration
    assert st("std::vector<int>::iterator i;") == ""

    # TODO: Handle the existing ones with Python equivalents
    # assert st("std::...")

    # FIXME: Maybe a better interpretation?
    # assert st("QDebug operator<<(QDebug dbg, const Message &message)") == "def __str__(self):"

    # TODO: Maybe play with the format?
    # assert st('m_o.append(tr("version: %1.%2").arg(a).arg(b))')
    #           == 'm_o.append(tr("version: {1}.{2}".format(a, b)'


def test_lambdas():
    # QtConcurrent::blockingMap(vector, [](int &x) { x *= 2; });

    # QList<QImage> collage = QtConcurrent::mappedReduced(images,
    #        [&size](const QImage &image) {
    #            return image.scaled(size, size);
    #        },
    #        addToCollage
    #   ).results();
    pass


def test_switch_case():
    source = """switch (v) {
case 1:
    f1();
    break;
case ClassName::EnumValue:
    f2();
    break;
default:
    f3();
    break;
}
"""
    expected = """
if v == 1:
    f1()
    break
elif v == ClassName.EnumValue:
    f2()
    break
else:
    f3()
    break

"""

    assert multi_st(source) == expected


def test_std_function():
    # std::function<QImage(const QImage &)> scale = [](const QImage &img) {
    pass



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations


module_classes = {
    "Qt3DAnimation": [
        "QAbstractAnimation",
        "QAbstractAnimationClip",
        "QAbstractChannelMapping",
        "QAbstractClipAnimator",
        "QAbstractClipBlendNode",
        "QAdditiveClipBlend",
        "QAnimationAspect",
        "QAnimationCallback",
        "QAnimationClip",
        "QAnimationClipLoader",
        "QAnimationController",
        "QAnimationGroup",
        "QBlendedClipAnimator",
        "QClipAnimator",
        "QClock",
        "QKeyFrame",
        "QKeyframeAnimation",
        "QLerpClipBlend",
        "QMorphTarget",
        "QMorphingAnimation",
        "QSkeletonMapping",
        "QVertexBlendAnimation",
    ],
    "Qt3DCore": [
        "QAbstractAspect",
        "QAbstractSkeleton",
        "QArmature",
        "QAspectEngine",
        "QAspectJob",
        "QBackendNode",
        "QComponent",
        "QComponentAddedChange",
        "QComponentRemovedChange",
        "QDynamicPropertyUpdatedChange",
        "QEntity",
        "QJoint",
        "QNode",
        "QNodeCommand",
        "QNodeCreatedChangeBase",
        "QNodeDestroyedChange",
        "QNodeId",
        "QNodeIdTypePair",
        "QPropertyNodeAddedChange",
        "QPropertyNodeRemovedChange",
        "QPropertyUpdatedChange",
        "QPropertyUpdatedChangeBase",
        "QPropertyValueAddedChange",
        "QPropertyValueAddedChangeBase",
        "QPropertyValueRemovedChange",
        "QPropertyValueRemovedChangeBase",
        "QSceneChange",
        "QSkeleton",
        "QSkeletonLoader",
        "QStaticPropertyUpdatedChangeBase",
        "QStaticPropertyValueAddedChangeBase",
        "QStaticPropertyValueRemovedChangeBase",
        "QTransform",
    ],
    "Qt3DExtras": [
        "QAbstractCameraController",
        "QAbstractSpriteSheet",
        "QConeGeometry",
        "QConeMesh",
        "QCuboidGeometry",
        "QCuboidMesh",
        "QCylinderGeometry",
        "QCylinderMesh",
        "QDiffuseMapMaterial",
        "QDiffuseSpecularMapMaterial",
        "QDiffuseSpecularMaterial",
        "QExtrudedTextGeometry",
        "QExtrudedTextMesh",
        "QFirstPersonCameraController",
        "QForwardRenderer",
        "QGoochMaterial",
        "QMetalRoughMaterial",
        "QMorphPhongMaterial",
        "QNormalDiffuseMapAlphaMaterial",
        "QNormalDiffuseMapMaterial",
        "QNormalDiffuseSpecularMapMaterial",
        "QOrbitCameraController",
        "QPerVertexColorMaterial",
        "QPhongAlphaMaterial",
        "QPhongMaterial",
        "QPlaneGeometry",
        "QPlaneMesh",
        "QSkyboxEntity",
        "QSphereGeometry",
        "QSphereMesh",
        "QSpriteGrid",
        "QSpriteSheet",
        "QSpriteSheetItem",
        "QText2DEntity",
        "QTextureMaterial",
        "QTorusGeometry",
        "QTorusMesh",
        "Qt3DWindow",
    ],
    "Qt3DInput": [
        "QAbstractActionInput",
        "QAbstractAxisInput",
        "QAbstractPhysicalDevice",
        "QAction",
        "QActionInput",
        "QAnalogAxisInput",
        "QAxis",
        "QAxisAccumulator",
        "QAxisSetting",
        "QButtonAxisInput",
        "QInputAspect",
        "QInputChord",
        "QInputSequence",
        "QInputSettings",
        "QKeyEvent",
        "QKeyboardDevice",
        "QKeyboardHandler",
        "QLogicalDevice",
        "QMouseDevice",
        "QMouseEvent",
        "QMouseHandler",
        "QWheelEvent",
    ],
    "Qt3DLogic": ["QFrameAction", "QLogicAspect"],
    "Qt3DRender": [
        "QAbstractFunctor",
        "QAbstractLight",
        "QAbstractRayCaster",
        "QAbstractTexture",
        "QAbstractTextureImage",
        "QAlphaCoverage",
        "QAlphaTest",
        "QAttribute",
        "QBlendEquation",
        "QBlendEquationArguments",
        "QBlitFramebuffer",
        "QBuffer",
        "QBufferCapture",
        "QBufferDataGenerator",
        "QCamera",
        "QCameraLens",
        "QCameraSelector",
        "QClearBuffers",
        "QClipPlane",
        "QColorMask",
        "QComputeCommand",
        "QCullFace",
        "QDepthTest",
        "QDirectionalLight",
        "QDispatchCompute",
        "QDithering",
        "QEffect",
        "QEnvironmentLight",
        "QFilterKey",
        "QFrameGraphNode",
        "QFrameGraphNodeCreatedChangeBase",
        "QFrontFace",
        "QFrustumCulling",
        "QGeometry",
        "QGeometryFactory",
        "QGeometryRenderer",
        "QGraphicsApiFilter",
        "QLayer",
        "QLayerFilter",
        "QLevelOfDetail",
        "QLevelOfDetailBoundingSphere",
        "QLevelOfDetailSwitch",
        "QLineWidth",
        "QMaterial",
        "QMemoryBarrier",
        "QMesh",
        "QMultiSampleAntiAliasing",
        "QNoDepthMask",
        "QNoDraw",
        "QNoPicking",
        "QObjectPicker",
        "QPaintedTextureImage",
        "QParameter",
        "QPickEvent",
        "QPickLineEvent",
        "QPickPointEvent",
        "QPickTriangleEvent",
        "QPickingSettings",
        "QPointLight",
        "QPointSize",
        "QPolygonOffset",
        "QProximityFilter",
        "QRayCaster",
        "QRayCasterHit",
        "QRenderAspect",
        "QRenderCapabilities",
        "QRenderCapture",
        "QRenderCaptureReply",
        "QRenderPass",
        "QRenderPassFilter",
        "QRenderSettings",
        "QRenderState",
        "QRenderStateSet",
        "QRenderSurfaceSelector",
        "QRenderTarget",
        "QRenderTargetOutput",
        "QRenderTargetSelector",
        "QSceneLoader",
        "QScissorTest",
        "QScreenRayCaster",
        "QSeamlessCubemap",
        "QSetFence",
        "QShaderData",
        "QShaderImage",
        "QShaderProgram",
        "QShaderProgramBuilder",
        "QSharedGLTexture",
        "QSortPolicy",
        "QSpotLight",
        "QStencilMask",
        "QStencilOperation",
        "QStencilOperationArguments",
        "QStencilTest",
        "QStencilTestArguments",
        "QTechnique",
        "QTechniqueFilter",
        "QTexture1D",
        "QTexture1DArray",
        "QTexture2D",
        "QTexture2DArray",
        "QTexture2DMultisample",
        "QTexture2DMultisampleArray",
        "QTexture3D",
        "QTextureBuffer",
        "QTextureCubeMap",
        "QTextureCubeMapArray",
        "QTextureData",
        "QTextureGenerator",
        "QTextureImage",
        "QTextureImageData",
        "QTextureImageDataGenerator",
        "QTextureLoader",
        "QTextureRectangle",
        "QTextureWrapMode",
        "QViewport",
        "QWaitFence",
    ],
    "QtCharts": [
        "QAbstractAxis",
        "QAbstractBarSeries",
        "QAbstractSeries",
        "QAreaLegendMarker",
        "QAreaSeries",
        "QBarCategoryAxis",
        "QBarLegendMarker",
        "QBarModelMapper",
        "QBarSeries",
        "QBarSet",
        "QBoxPlotLegendMarker",
        "QBoxPlotModelMapper",
        "QBoxPlotSeries",
        "QBoxSet",
        "QCandlestickLegendMarker",
        "QCandlestickModelMapper",
        "QCandlestickSeries",
        "QCandlestickSet",
        "QCategoryAxis",
        "QChart",
        "QChartView",
        "QDateTimeAxis",
        "QHBarModelMapper",
        "QHBoxPlotModelMapper",
        "QHCandlestickModelMapper",
        "QHPieModelMapper",
        "QHXYModelMapper",
        "QHorizontalBarSeries",
        "QHorizontalPercentBarSeries",
        "QHorizontalStackedBarSeries",
        "QLegend",
        "QLegendMarker",
        "QLineSeries",
        "QLogValueAxis",
        "QPercentBarSeries",
        "QPieLegendMarker",
        "QPieModelMapper",
        "QPieSeries",
        "QPieSlice",
        "QPolarChart",
        "QScatterSeries",
        "QSplineSeries",
        "QStackedBarSeries",
        "QVBarModelMapper",
        "QVBoxPlotModelMapper",
        "QVCandlestickModelMapper",
        "QVPieModelMapper",
        "QVXYModelMapper",
        "QValueAxis",
        "QXYLegendMarker",
        "QXYModelMapper",
        "QXYSeries",
    ],
    "QtConcurrent": [
        "QFutureQString",
        "QFutureVoid",
        "QFutureWatcherQString",
        "QFutureWatcherVoid",
        "QtConcurrent",
    ],
    "QtCore": [
        "QAbstractAnimation",
        "QAbstractEventDispatcher",
        "QAbstractItemModel",
        "QAbstractListModel",
        "QAbstractNativeEventFilter",
        "QAbstractProxyModel",
        "QAbstractState",
        "QAbstractTableModel",
        "QAbstractTransition",
        "QAnimationGroup",
        "QBasicMutex",
        "QBasicTimer",
        "QBitArray",
        "QBuffer",
        "QByteArray",
        "QByteArrayMatcher",
        "QCalendar",
        "QCborArray",
        "QCborError",
        "QCborKnownTags",
        "QCborMap",
        "QCborParserError",
        "QCborSimpleType",
        "QCborStreamReader",
        "QCborStreamWriter",
        "QCborStringResultByteArray",
        "QCborStringResultString",
        "QCborValue",
        "QChildEvent",
        "QCollator",
        "QCollatorSortKey",
        "QCommandLineOption",
        "QCommandLineParser",
        "QConcatenateTablesProxyModel",
        "QCoreApplication",
        "QCryptographicHash",
        "QDataStream",
        "QDate",
        "QDateTime",
        "QDeadlineTimer",
        "QDir",
        "QDirIterator",
        "QDynamicPropertyChangeEvent",
        "QEasingCurve",
        "QElapsedTimer",
        "QEnum",
        "QEvent",
        "QEventLoop",
        "QEventTransition",
        "QFactoryInterface",
        "QFile",
        "QFileDevice",
        "QFileInfo",
        "QFileSelector",
        "QFileSystemWatcher",
        "QFinalState",
        "QFlag",
        "QFutureInterfaceBase",
        "QGenericArgument",
        "QGenericReturnArgument",
        "QHistoryState",
        "QIODevice",
        "QIdentityProxyModel",
        "QItemSelection",
        "QItemSelectionModel",
        "QItemSelectionRange",
        "QJsonArray",
        "QJsonDocument",
        "QJsonParseError",
        "QJsonValue",
        "QLibraryInfo",
        "QLine",
        "QLineF",
        "QLocale",
        "QLockFile",
        "QMargins",
        "QMarginsF",
        "QMessageAuthenticationCode",
        "QMessageLogContext",
        "QMetaClassInfo",
        "QMetaEnum",
        "QMetaMethod",
        "QMetaObject",
        "QMetaProperty",
        "QMimeData",
        "QMimeDatabase",
        "QMimeType",
        "QModelIndex",
        "QMutex",
        "QMutexLocker",
        "QObject",
        "QOperatingSystemVersion",
        "QParallelAnimationGroup",
        "QPauseAnimation",
        "QPersistentModelIndex",
        "QPluginLoader",
        "QPoint",
        "QPointF",
        "QProcess",
        "QProcessEnvironment",
        "QPropertyAnimation",
        "QRandomGenerator",
        "QRandomGenerator64",
        "QReadLocker",
        "QReadWriteLock",
        "QRect",
        "QRectF",
        "QRecursiveMutex",
        "QRegExp",
        "QRegularExpression",
        "QRegularExpressionMatch",
        "QRegularExpressionMatchIterator",
        "QResource",
        "QRunnable",
        "QSaveFile",
        "QSemaphore",
        "QSemaphoreReleaser",
        "QSequentialAnimationGroup",
        "QSettings",
        "QSignalBlocker",
        "QSignalMapper",
        "QSignalTransition",
        "QSize",
        "QSizeF",
        "QSocketDescriptor",
        "QSocketNotifier",
        "QSortFilterProxyModel",
        "QStandardPaths",
        "QState",
        "QStateMachine",
        "QStorageInfo",
        "QStringListModel",
        "QSysInfo",
        "QSystemSemaphore",
        "QT_TRANSLATE_NOOP",
        "QT_TRANSLATE_NOOP3",
        "QT_TRANSLATE_NOOP_UTF8",
        "QT_TR_NOOP",
        "QT_TR_NOOP_UTF8",
        "QTemporaryDir",
        "QTemporaryFile",
        "QTextBoundaryFinder",
        "QTextCodec",
        "QTextDecoder",
        "QTextEncoder",
        "QTextStream",
        "QTextStreamManipulator",
        "QThread",
        "QThreadPool",
        "QTime",
        "QTimeLine",
        "QTimeZone",
        "QTimer",
        "QTimerEvent",
        "QTranslator",
        "QTransposeProxyModel",
        "QUrl",
        "QUrlQuery",
        "QUuid",
        "QVariantAnimation",
        "QVersionNumber",
        "QWaitCondition",
        "QWriteLocker",
        "QXmlStreamAttribute",
        "QXmlStreamAttributes",
        "QXmlStreamEntityDeclaration",
        "QXmlStreamEntityResolver",
        "QXmlStreamNamespaceDeclaration",
        "QXmlStreamNotationDeclaration",
        "QXmlStreamReader",
        "QXmlStreamWriter",
        "Qt",
        "QtCriticalMsg",
        "QtDebugMsg",
        "QtFatalMsg",
        "QtInfoMsg",
        "QtMsgType",
        "QtSystemMsg",
        "QtWarningMsg",
    ],
    "QtDataVisualization": [
        "Q3DBars",
        "Q3DCamera",
        "Q3DInputHandler",
        "Q3DLight",
        "Q3DObject",
        "Q3DScatter",
        "Q3DScene",
        "Q3DSurface",
        "Q3DTheme",
        "QAbstract3DAxis",
        "QAbstract3DGraph",
        "QAbstract3DInputHandler",
        "QAbstract3DSeries",
        "QAbstractDataProxy",
        "QBar3DSeries",
        "QBarDataItem",
        "QBarDataProxy",
        "QCategory3DAxis",
        "QCustom3DItem",
        "QCustom3DLabel",
        "QCustom3DVolume",
        "QHeightMapSurfaceDataProxy",
        "QItemModelBarDataProxy",
        "QItemModelScatterDataProxy",
        "QItemModelSurfaceDataProxy",
        "QLogValue3DAxisFormatter",
        "QScatter3DSeries",
        "QScatterDataItem",
        "QScatterDataProxy",
        "QSurface3DSeries",
        "QSurfaceDataItem",
        "QSurfaceDataProxy",
        "QTouch3DInputHandler",
        "QValue3DAxis",
        "QValue3DAxisFormatter",
    ],
    "QtGui": [
        "QAbstractOpenGLFunctions",
        "QAbstractTextDocumentLayout",
        "QAccessible",
        "QAccessibleAnnouncementEvent",
        "QAccessibleAttributesInterface",
        "QAccessibleEditableTextInterface",
        "QAccessibleEvent",
        "QAccessibleInterface",
        "QAccessibleObject",
        "QAccessibleSelectionInterface",
        "QAccessibleStateChangeEvent",
        "QAccessibleTableCellInterface",
        "QAccessibleTableModelChangeEvent",
        "QAccessibleTextCursorEvent",
        "QAccessibleTextInsertEvent",
        "QAccessibleTextInterface",
        "QAccessibleTextRemoveEvent",
        "QAccessibleTextSelectionEvent",
        "QAccessibleTextUpdateEvent",
        "QAccessibleValueChangeEvent",
        "QAccessibleValueInterface",
        "QActionEvent",
        "QBackingStore",
        "QBitmap",
        "QBrush",
        "QClipboard",
        "QCloseEvent",
        "QColor",
        "QColorConstants",
        "QColorSpace",
        "QConicalGradient",
        "QContextMenuEvent",
        "QCursor",
        "QDesktopServices",
        "QDoubleValidator",
        "QDrag",
        "QDragEnterEvent",
        "QDragLeaveEvent",
        "QDragMoveEvent",
        "QDropEvent",
        "QEnterEvent",
        "QExposeEvent",
        "QFileOpenEvent",
        "QFocusEvent",
        "QFont",
        "QFontDatabase",
        "QFontInfo",
        "QFontMetrics",
        "QFontMetricsF",
        "QGradient",
        "QGuiApplication",
        "QHelpEvent",
        "QHideEvent",
        "QHoverEvent",
        "QIcon",
        "QIconDragEvent",
        "QIconEngine",
        "QImage",
        "QImageIOHandler",
        "QImageReader",
        "QImageWriter",
        "QInputEvent",
        "QInputMethod",
        "QInputMethodEvent",
        "QInputMethodQueryEvent",
        "QIntValidator",
        "QKeyEvent",
        "QKeySequence",
        "QLinearGradient",
        "QMatrix",
        "QMatrix2x2",
        "QMatrix2x3",
        "QMatrix2x4",
        "QMatrix3x2",
        "QMatrix3x3",
        "QMatrix3x4",
        "QMatrix4x2",
        "QMatrix4x3",
        "QMatrix4x4",
        "QMouseEvent",
        "QMoveEvent",
        "QMovie",
        "QNativeGestureEvent",
        "QOffscreenSurface",
        "QOpenGLBuffer",
        "QOpenGLContext",
        "QOpenGLContextGroup",
        "QOpenGLDebugLogger",
        "QOpenGLDebugMessage",
        "QOpenGLExtraFunctions",
        "QOpenGLFramebufferObject",
        "QOpenGLFramebufferObjectFormat",
        "QOpenGLFunctions",
        "QOpenGLPixelTransferOptions",
        "QOpenGLShader",
        "QOpenGLShaderProgram",
        "QOpenGLTexture",
        "QOpenGLTextureBlitter",
        "QOpenGLTimeMonitor",
        "QOpenGLTimerQuery",
        "QOpenGLVersionProfile",
        "QOpenGLVertexArrayObject",
        "QOpenGLWindow",
        "QPageLayout",
        "QPageSize",
        "QPagedPaintDevice",
        "QPaintDevice",
        "QPaintDeviceWindow",
        "QPaintEngine",
        "QPaintEngineState",
        "QPaintEvent",
        "QPainter",
        "QPainterPath",
        "QPainterPathStroker",
        "QPalette",
        "QPdfWriter",
        "QPen",
        "QPicture",
        "QPictureIO",
        "QPixelFormat",
        "QPixmap",
        "QPixmapCache",
        "QPointingDeviceUniqueId",
        "QPolygon",
        "QPolygonF",
        "QPyTextObject",
        "QQuaternion",
        "QRadialGradient",
        "QRasterWindow",
        "QRawFont",
        "QRegExpValidator",
        "QRegion",
        "QRegularExpressionValidator",
        "QResizeEvent",
        "QScreen",
        "QScrollEvent",
        "QScrollPrepareEvent",
        "QSessionManager",
        "QShortcutEvent",
        "QShowEvent",
        "QStandardItem",
        "QStandardItemModel",
        "QStaticText",
        "QStatusTipEvent",
        "QStyleHints",
        "QSurface",
        "QSurfaceFormat",
        "QSyntaxHighlighter",
        "QTabletEvent",
        "QTextBlock",
        "QTextBlockFormat",
        "QTextBlockGroup",
        "QTextBlockUserData",
        "QTextCharFormat",
        "QTextCursor",
        "QTextDocument",
        "QTextDocumentFragment",
        "QTextDocumentWriter",
        "QTextFormat",
        "QTextFragment",
        "QTextFrame",
        "QTextFrameFormat",
        "QTextImageFormat",
        "QTextInlineObject",
        "QTextItem",
        "QTextLayout",
        "QTextLength",
        "QTextLine",
        "QTextList",
        "QTextListFormat",
        "QTextObject",
        "QTextObjectInterface",
        "QTextOption",
        "QTextTable",
        "QTextTableCell",
        "QTextTableCellFormat",
        "QTextTableFormat",
        "QToolBarChangeEvent",
        "QTouchDevice",
        "QTouchEvent",
        "QTransform",
        "QValidator",
        "QVector2D",
        "QVector3D",
        "QVector4D",
        "QWhatsThisClickedEvent",
        "QWheelEvent",
        "QWindow",
        "QWindowStateChangeEvent",
        "Qt",
    ],
    "QtHelp": [
        "QCompressedHelpInfo",
        "QHelpContentItem",
        "QHelpContentModel",
        "QHelpContentWidget",
        "QHelpEngine",
        "QHelpEngineCore",
        "QHelpFilterData",
        "QHelpFilterEngine",
        "QHelpFilterSettingsWidget",
        "QHelpIndexModel",
        "QHelpIndexWidget",
        "QHelpLink",
        "QHelpSearchEngine",
        "QHelpSearchQuery",
        "QHelpSearchQueryWidget",
        "QHelpSearchResult",
        "QHelpSearchResultWidget",
    ],
    "QtLocation": [
        "QGeoCodeReply",
        "QGeoCodingManager",
        "QGeoCodingManagerEngine",
        "QGeoManeuver",
        "QGeoRoute",
        "QGeoRouteReply",
        "QGeoRouteRequest",
        "QGeoRouteSegment",
        "QGeoRoutingManager",
        "QGeoRoutingManagerEngine",
        "QGeoServiceProvider",
        "QGeoServiceProviderFactory",
        "QGeoServiceProviderFactoryV2",
        "QPlace",
        "QPlaceAttribute",
        "QPlaceCategory",
        "QPlaceContactDetail",
        "QPlaceContent",
        "QPlaceContentReply",
        "QPlaceContentRequest",
        "QPlaceDetailsReply",
        "QPlaceEditorial",
        "QPlaceIcon",
        "QPlaceIdReply",
        "QPlaceImage",
        "QPlaceManager",
        "QPlaceManagerEngine",
        "QPlaceMatchReply",
        "QPlaceMatchRequest",
        "QPlaceProposedSearchResult",
        "QPlaceRatings",
        "QPlaceReply",
        "QPlaceResult",
        "QPlaceReview",
        "QPlaceSearchReply",
        "QPlaceSearchRequest",
        "QPlaceSearchResult",
        "QPlaceSearchSuggestionReply",
        "QPlaceSupplier",
        "QPlaceUser",
    ],
    "QtMultimedia": [
        "QAbstractAudioDeviceInfo",
        "QAbstractAudioInput",
        "QAbstractAudioOutput",
        "QAbstractVideoBuffer",
        "QAbstractVideoFilter",
        "QAbstractVideoSurface",
        "QAudio",
        "QAudioBuffer",
        "QAudioDecoder",
        "QAudioDecoderControl",
        "QAudioDeviceInfo",
        "QAudioEncoderSettings",
        "QAudioEncoderSettingsControl",
        "QAudioFormat",
        "QAudioInput",
        "QAudioInputSelectorControl",
        "QAudioOutput",
        "QAudioOutputSelectorControl",
        "QAudioProbe",
        "QAudioRecorder",
        "QAudioRoleControl",
        "QCamera",
        "QCameraCaptureBufferFormatControl",
        "QCameraCaptureDestinationControl",
        "QCameraControl",
        "QCameraExposure",
        "QCameraExposureControl",
        "QCameraFeedbackControl",
        "QCameraFlashControl",
        "QCameraFocus",
        "QCameraFocusControl",
        "QCameraFocusZone",
        "QCameraImageCapture",
        "QCameraImageCaptureControl",
        "QCameraImageProcessing",
        "QCameraImageProcessingControl",
        "QCameraInfo",
        "QCameraInfoControl",
        "QCameraLocksControl",
        "QCameraViewfinderSettings",
        "QCameraViewfinderSettingsControl",
        "QCameraViewfinderSettingsControl2",
        "QCameraZoomControl",
        "QCustomAudioRoleControl",
        "QImageEncoderControl",
        "QImageEncoderSettings",
        "QMediaAudioProbeControl",
        "QMediaAvailabilityControl",
        "QMediaBindableInterface",
        "QMediaContainerControl",
        "QMediaContent",
        "QMediaControl",
        "QMediaGaplessPlaybackControl",
        "QMediaNetworkAccessControl",
        "QMediaObject",
        "QMediaPlayer",
        "QMediaPlayerControl",
        "QMediaPlaylist",
        "QMediaRecorder",
        "QMediaRecorderControl",
        "QMediaResource",
        "QMediaService",
        "QMediaServiceCameraInfoInterface",
        "QMediaServiceDefaultDeviceInterface",
        "QMediaServiceFeaturesInterface",
        "QMediaServiceProviderHint",
        "QMediaServiceSupportedDevicesInterface",
        "QMediaServiceSupportedFormatsInterface",
        "QMediaStreamsControl",
        "QMediaTimeInterval",
        "QMediaTimeRange",
        "QMediaVideoProbeControl",
        "QMetaDataReaderControl",
        "QMetaDataWriterControl",
        "QMultimedia",
        "QRadioData",
        "QRadioDataControl",
        "QRadioTuner",
        "QRadioTunerControl",
        "QSound",
        "QSoundEffect",
        "QVideoDeviceSelectorControl",
        "QVideoEncoderSettings",
        "QVideoEncoderSettingsControl",
        "QVideoFilterRunnable",
        "QVideoFrame",
        "QVideoProbe",
        "QVideoRendererControl",
        "QVideoSurfaceFormat",
        "QVideoWindowControl",
    ],
    "QtMultimediaWidgets": [
        "QCameraViewfinder",
        "QGraphicsVideoItem",
        "QVideoWidget",
        "QVideoWidgetControl",
    ],
    "QtNetwork": [
        "QAbstractNetworkCache",
        "QAbstractSocket",
        "QAuthenticator",
        "QDnsDomainNameRecord",
        "QDnsHostAddressRecord",
        "QDnsLookup",
        "QDnsMailExchangeRecord",
        "QDnsServiceRecord",
        "QDnsTextRecord",
        "QDtls",
        "QDtlsError",
        "QHostAddress",
        "QHostInfo",
        "QHstsPolicy",
        "QHttpMultiPart",
        "QHttpPart",
        "QIPv6Address",
        "QLocalServer",
        "QLocalSocket",
        "QNetworkAccessManager",
        "QNetworkAddressEntry",
        "QNetworkCacheMetaData",
        "QNetworkConfiguration",
        "QNetworkConfigurationManager",
        "QNetworkCookie",
        "QNetworkCookieJar",
        "QNetworkDatagram",
        "QNetworkDiskCache",
        "QNetworkInterface",
        "QNetworkProxy",
        "QNetworkProxyFactory",
        "QNetworkProxyQuery",
        "QNetworkReply",
        "QNetworkRequest",
        "QNetworkSession",
        "QOcspCertificateStatus",
        "QOcspResponse",
        "QOcspRevocationReason",
        "QPasswordDigestor",
        "QSsl",
        "QSslCertificate",
        "QSslCertificateExtension",
        "QSslCipher",
        "QSslConfiguration",
        "QSslDiffieHellmanParameters",
        "QSslError",
        "QSslKey",
        "QSslPreSharedKeyAuthenticator",
        "QSslSocket",
        "QTcpServer",
        "QTcpSocket",
        "QUdpSocket",
    ],
    "QtOpenGL": [
        "QGL",
        "QGLBuffer",
        "QGLColormap",
        "QGLContext",
        "QGLFormat",
        "QGLFramebufferObject",
        "QGLFramebufferObjectFormat",
        "QGLPixelBuffer",
        "QGLShader",
        "QGLShaderProgram",
        "QGLWidget",
    ],
    "QtOpenGLFunctions": [
        "QOpenGLFunctions_1_0",
        "QOpenGLFunctions_1_1",
        "QOpenGLFunctions_1_2",
        "QOpenGLFunctions_1_3",
        "QOpenGLFunctions_1_4",
        "QOpenGLFunctions_1_5",
        "QOpenGLFunctions_2_0",
        "QOpenGLFunctions_2_1",
        "QOpenGLFunctions_3_0",
        "QOpenGLFunctions_3_1",
        "QOpenGLFunctions_3_2_Compatibility",
        "QOpenGLFunctions_3_2_Core",
        "QOpenGLFunctions_3_3_Compatibility",
        "QOpenGLFunctions_3_3_Core",
        "QOpenGLFunctions_4_0_Compatibility",
        "QOpenGLFunctions_4_0_Core",
        "QOpenGLFunctions_4_1_Compatibility",
        "QOpenGLFunctions_4_1_Core",
        "QOpenGLFunctions_4_2_Compatibility",
        "QOpenGLFunctions_4_2_Core",
        "QOpenGLFunctions_4_3_Compatibility",
        "QOpenGLFunctions_4_3_Core",
        "QOpenGLFunctions_4_4_Compatibility",
        "QOpenGLFunctions_4_4_Core",
        "QOpenGLFunctions_4_5_Compatibility",
        "QOpenGLFunctions_4_5_Core",
    ],
    "QtPositioning": [
        "QGeoAddress",
        "QGeoAreaMonitorInfo",
        "QGeoAreaMonitorSource",
        "QGeoCircle",
        "QGeoCoordinate",
        "QGeoLocation",
        "QGeoPath",
        "QGeoPolygon",
        "QGeoPositionInfo",
        "QGeoPositionInfoSource",
        "QGeoPositionInfoSourceFactory",
        "QGeoRectangle",
        "QGeoSatelliteInfo",
        "QGeoSatelliteInfoSource",
        "QGeoShape",
        "QNmeaPositionInfoSource",
    ],
    "QtPrintSupport": [
        "QAbstractPrintDialog",
        "QPageSetupDialog",
        "QPrintDialog",
        "QPrintEngine",
        "QPrintPreviewDialog",
        "QPrintPreviewWidget",
        "QPrinter",
        "QPrinterInfo",
    ],
    "QtQml": [
        "QJSEngine",
        "QJSValue",
        "QJSValueIterator",
        "QQmlAbstractUrlInterceptor",
        "QQmlApplicationEngine",
        "QQmlComponent",
        "QQmlContext",
        "QQmlDebuggingEnabler",
        "QQmlEngine",
        "QQmlError",
        "QQmlExpression",
        "QQmlExtensionInterface",
        "QQmlExtensionPlugin",
        "QQmlFile",
        "QQmlFileSelector",
        "QQmlImageProviderBase",
        "QQmlIncubationController",
        "QQmlIncubator",
        "QQmlListReference",
        "QQmlNetworkAccessManagerFactory",
        "QQmlParserStatus",
        "QQmlProperty",
        "QQmlPropertyMap",
        "QQmlPropertyValueSource",
        "QQmlScriptString",
        "QQmlTypesExtensionInterface",
        "QtQml",
    ],
    "QtQuick": [
        "QQuickAsyncImageProvider",
        "QQuickFramebufferObject",
        "QQuickImageProvider",
        "QQuickImageResponse",
        "QQuickItem",
        "QQuickItemGrabResult",
        "QQuickPaintedItem",
        "QQuickRenderControl",
        "QQuickTextDocument",
        "QQuickTextureFactory",
        "QQuickTransform",
        "QQuickView",
        "QQuickWindow",
        "QSGAbstractRenderer",
        "QSGBasicGeometryNode",
        "QSGClipNode",
        "QSGDynamicTexture",
        "QSGEngine",
        "QSGGeometry",
        "QSGGeometryNode",
        "QSGMaterialType",
        "QSGNode",
        "QSGOpacityNode",
        "QSGSimpleRectNode",
        "QSGSimpleTextureNode",
        "QSGTexture",
        "QSGTextureProvider",
        "QSGTransformNode",
        "QSharedPointer<QQuickItemGrabResult >",
    ],
    "QtQuickControls2": ["QQuickStyle"],
    "QtQuickWidgets": ["QQuickWidget"],
    "QtRemoteObjects": [
        "QAbstractItemModelReplica",
        "QRemoteObjectAbstractPersistedStore",
        "QRemoteObjectDynamicReplica",
        "QRemoteObjectHost",
        "QRemoteObjectHostBase",
        "QRemoteObjectNode",
        "QRemoteObjectPendingCall",
        "QRemoteObjectPendingCallWatcher",
        "QRemoteObjectRegistry",
        "QRemoteObjectRegistryHost",
        "QRemoteObjectReplica",
        "QRemoteObjectSettingsStore",
        "QRemoteObjectSourceLocationInfo",
    ],
    "QtScript": [
        "QScriptClass",
        "QScriptClassPropertyIterator",
        "QScriptContext",
        "QScriptContextInfo",
        "QScriptEngine",
        "QScriptEngineAgent",
        "QScriptExtensionInterface",
        "QScriptExtensionPlugin",
        "QScriptProgram",
        "QScriptString",
        "QScriptValue",
        "QScriptValueIterator",
        "QScriptable",
    ],
    "QtScriptTools": ["QScriptEngineDebugger"],
    "QtScxml": [
        "QScxmlCompiler",
        "QScxmlCppDataModel",
        "QScxmlDataModel",
        "QScxmlDynamicScxmlServiceFactory",
        "QScxmlEcmaScriptDataModel",
        "QScxmlError",
        "QScxmlEvent",
        "QScxmlExecutableContent",
        "QScxmlInvokableService",
        "QScxmlInvokableServiceFactory",
        "QScxmlNullDataModel",
        "QScxmlStateMachine",
        "QScxmlStaticScxmlServiceFactory",
        "QScxmlTableData",
    ],
    "QtSensors": [
        "QAccelerometer",
        "QAccelerometerFilter",
        "QAccelerometerReading",
        "QAltimeter",
        "QAltimeterFilter",
        "QAltimeterReading",
        "QAmbientLightFilter",
        "QAmbientLightReading",
        "QAmbientLightSensor",
        "QAmbientTemperatureFilter",
        "QAmbientTemperatureReading",
        "QAmbientTemperatureSensor",
        "QCompass",
        "QCompassFilter",
        "QCompassReading",
        "QDistanceFilter",
        "QDistanceReading",
        "QDistanceSensor",
        "QGyroscope",
        "QGyroscopeFilter",
        "QGyroscopeReading",
        "QHolsterFilter",
        "QHolsterReading",
        "QHolsterSensor",
        "QHumidityFilter",
        "QHumidityReading",
        "QHumiditySensor",
        "QIRProximityFilter",
        "QIRProximityReading",
        "QIRProximitySensor",
        "QLidFilter",
        "QLidReading",
        "QLidSensor",
        "QLightFilter",
        "QLightReading",
        "QLightSensor",
        "QMagnetometer",
        "QMagnetometerFilter",
        "QMagnetometerReading",
        "QOrientationFilter",
        "QOrientationReading",
        "QOrientationSensor",
        "QPressureFilter",
        "QPressureReading",
        "QPressureSensor",
        "QProximityFilter",
        "QProximityReading",
        "QProximitySensor",
        "QRotationFilter",
        "QRotationReading",
        "QRotationSensor",
        "QSensor",
        "QSensorBackend",
        "QSensorBackendFactory",
        "QSensorChangesInterface",
        "QSensorFilter",
        "QSensorGestureManager",
        "QSensorGesturePluginInterface",
        "QSensorGestureRecognizer",
        "QSensorManager",
        "QSensorPluginInterface",
        "QSensorReading",
        "QTapFilter",
        "QTapReading",
        "QTapSensor",
        "QTiltFilter",
        "QTiltReading",
        "QTiltSensor",
    ],
    "QtSerialPort": ["QSerialPort", "QSerialPortInfo"],
    "QtSql": [
        "QSql",
        "QSqlDatabase",
        "QSqlDriver",
        "QSqlDriverCreatorBase",
        "QSqlError",
        "QSqlField",
        "QSqlIndex",
        "QSqlQuery",
        "QSqlQueryModel",
        "QSqlRecord",
        "QSqlRelation",
        "QSqlRelationalDelegate",
        "QSqlRelationalTableModel",
        "QSqlResult",
        "QSqlTableModel",
    ],
    "QtSvg": ["QGraphicsSvgItem", "QSvgGenerator", "QSvgRenderer", "QSvgWidget"],
    "QtTest": ["QTest"],
    "QtTextToSpeech": ["QTextToSpeech", "QTextToSpeechEngine", "QVoice"],
    "QtUiTools": ["QUiLoader"],
    "QtWebChannel": ["QWebChannel", "QWebChannelAbstractTransport"],
    "QtWebEngine": [],
    "QtWebEngineCore": [
        "QWebEngineCookieStore",
        "QWebEngineHttpRequest",
        "QWebEngineUrlRequestInfo",
        "QWebEngineUrlRequestInterceptor",
        "QWebEngineUrlRequestJob",
        "QWebEngineUrlScheme",
        "QWebEngineUrlSchemeHandler",
    ],
    "QtWebEngineWidgets": [
        "QWebEngineCertificateError",
        "QWebEngineContextMenuData",
        "QWebEngineDownloadItem",
        "QWebEngineFullScreenRequest",
        "QWebEngineHistory",
        "QWebEngineHistoryItem",
        "QWebEnginePage",
        "QWebEngineProfile",
        "QWebEngineScript",
        "QWebEngineScriptCollection",
        "QWebEngineSettings",
        "QWebEngineView",
    ],
    "QtWebSockets": [
        "QMaskGenerator",
        "QWebSocket",
        "QWebSocketCorsAuthenticator",
        "QWebSocketProtocol",
        "QWebSocketServer",
    ],
    "QtWidgets": [
        "QAbstractButton",
        "QAbstractGraphicsShapeItem",
        "QAbstractItemDelegate",
        "QAbstractItemView",
        "QAbstractScrollArea",
        "QAbstractSlider",
        "QAbstractSpinBox",
        "QAccessibleWidget",
        "QAction",
        "QActionGroup",
        "QApplication",
        "QBoxLayout",
        "QButtonGroup",
        "QCalendarWidget",
        "QCheckBox",
        "QColorDialog",
        "QColormap",
        "QColumnView",
        "QComboBox",
        "QCommandLinkButton",
        "QCommonStyle",
        "QCompleter",
        "QDataWidgetMapper",
        "QDateEdit",
        "QDateTimeEdit",
        "QDesktopWidget",
        "QDial",
        "QDialog",
        "QDialogButtonBox",
        "QDirModel",
        "QDockWidget",
        "QDoubleSpinBox",
        "QErrorMessage",
        "QFileDialog",
        "QFileIconProvider",
        "QFileSystemModel",
        "QFocusFrame",
        "QFontComboBox",
        "QFontDialog",
        "QFormLayout",
        "QFrame",
        "QGesture",
        "QGestureEvent",
        "QGestureRecognizer",
        "QGraphicsAnchor",
        "QGraphicsAnchorLayout",
        "QGraphicsBlurEffect",
        "QGraphicsColorizeEffect",
        "QGraphicsDropShadowEffect",
        "QGraphicsEffect",
        "QGraphicsEllipseItem",
        "QGraphicsGridLayout",
        "QGraphicsItem",
        "QGraphicsItemAnimation",
        "QGraphicsItemGroup",
        "QGraphicsLayout",
        "QGraphicsLayoutItem",
        "QGraphicsLineItem",
        "QGraphicsLinearLayout",
        "QGraphicsObject",
        "QGraphicsOpacityEffect",
        "QGraphicsPathItem",
        "QGraphicsPixmapItem",
        "QGraphicsPolygonItem",
        "QGraphicsProxyWidget",
        "QGraphicsRectItem",
        "QGraphicsRotation",
        "QGraphicsScale",
        "QGraphicsScene",
        "QGraphicsSceneContextMenuEvent",
        "QGraphicsSceneDragDropEvent",
        "QGraphicsSceneEvent",
        "QGraphicsSceneHelpEvent",
        "QGraphicsSceneHoverEvent",
        "QGraphicsSceneMouseEvent",
        "QGraphicsSceneMoveEvent",
        "QGraphicsSceneResizeEvent",
        "QGraphicsSceneWheelEvent",
        "QGraphicsSimpleTextItem",
        "QGraphicsTextItem",
        "QGraphicsTransform",
        "QGraphicsView",
        "QGraphicsWidget",
        "QGridLayout",
        "QGroupBox",
        "QHBoxLayout",
        "QHeaderView",
        "QInputDialog",
        "QItemDelegate",
        "QItemEditorCreatorBase",
        "QItemEditorFactory",
        "QKeyEventTransition",
        "QKeySequenceEdit",
        "QLCDNumber",
        "QLabel",
        "QLayout",
        "QLayoutItem",
        "QLineEdit",
        "QListView",
        "QListWidget",
        "QListWidgetItem",
        "QMainWindow",
        "QMdiArea",
        "QMdiSubWindow",
        "QMenu",
        "QMenuBar",
        "QMessageBox",
        "QMouseEventTransition",
        "QOpenGLWidget",
        "QPanGesture",
        "QPinchGesture",
        "QPlainTextDocumentLayout",
        "QPlainTextEdit",
        "QProgressBar",
        "QProgressDialog",
        "QProxyStyle",
        "QPushButton",
        "QRadioButton",
        "QRubberBand",
        "QScrollArea",
        "QScrollBar",
        "QScroller",
        "QScrollerProperties",
        "QShortcut",
        "QSizeGrip",
        "QSizePolicy",
        "QSlider",
        "QSpacerItem",
        "QSpinBox",
        "QSplashScreen",
        "QSplitter",
        "QSplitterHandle",
        "QStackedLayout",
        "QStackedWidget",
        "QStatusBar",
        "QStyle",
        "QStyleFactory",
        "QStyleHintReturn",
        "QStyleHintReturnMask",
        "QStyleHintReturnVariant",
        "QStyleOption",
        "QStyleOptionButton",
        "QStyleOptionComboBox",
        "QStyleOptionComplex",
        "QStyleOptionDockWidget",
        "QStyleOptionFocusRect",
        "QStyleOptionFrame",
        "QStyleOptionGraphicsItem",
        "QStyleOptionGroupBox",
        "QStyleOptionHeader",
        "QStyleOptionMenuItem",
        "QStyleOptionProgressBar",
        "QStyleOptionRubberBand",
        "QStyleOptionSizeGrip",
        "QStyleOptionSlider",
        "QStyleOptionSpinBox",
        "QStyleOptionTab",
        "QStyleOptionTabBarBase",
        "QStyleOptionTabWidgetFrame",
        "QStyleOptionTitleBar",
        "QStyleOptionToolBar",
        "QStyleOptionToolBox",
        "QStyleOptionToolButton",
        "QStyleOptionViewItem",
        "QStylePainter",
        "QStyledItemDelegate",
        "QSwipeGesture",
        "QSystemTrayIcon",
        "QTabBar",
        "QTabWidget",
        "QTableView",
        "QTableWidget",
        "QTableWidgetItem",
        "QTableWidgetSelectionRange",
        "QTapAndHoldGesture",
        "QTapGesture",
        "QTextBrowser",
        "QTextEdit",
        "QTileRules",
        "QTimeEdit",
        "QToolBar",
        "QToolBox",
        "QToolButton",
        "QToolTip",
        "QTreeView",
        "QTreeWidget",
        "QTreeWidgetItem",
        "QTreeWidgetItemIterator",
        "QUndoCommand",
        "QUndoGroup",
        "QUndoStack",
        "QUndoView",
        "QVBoxLayout",
        "QWhatsThis",
        "QWidget",
        "QWidgetAction",
        "QWidgetItem",
        "QWizard",
        "QWizardPage",
    ],
    "QtX11Extras": ["QX11Info"],
    "QtXml": [
        "QDomAttr",
        "QDomCDATASection",
        "QDomCharacterData",
        "QDomComment",
        "QDomDocument",
        "QDomDocumentFragment",
        "QDomDocumentType",
        "QDomElement",
        "QDomEntity",
        "QDomEntityReference",
        "QDomImplementation",
        "QDomNamedNodeMap",
        "QDomNode",
        "QDomNodeList",
        "QDomNotation",
        "QDomProcessingInstruction",
        "QDomText",
        "QXmlAttributes",
        "QXmlContentHandler",
        "QXmlDTDHandler",
        "QXmlDeclHandler",
        "QXmlDefaultHandler",
        "QXmlEntityResolver",
        "QXmlErrorHandler",
        "QXmlInputSource",
        "QXmlLexicalHandler",
        "QXmlLocator",
        "QXmlNamespaceSupport",
        "QXmlParseException",
        "QXmlReader",
        "QXmlSimpleReader",
    ],
    "QtXmlPatterns": [
        "QAbstractMessageHandler",
        "QAbstractUriResolver",
        "QAbstractXmlNodeModel",
        "QAbstractXmlReceiver",
        "QSourceLocation",
        "QXmlFormatter",
        "QXmlItem",
        "QXmlName",
        "QXmlNamePool",
        "QXmlNodeModelIndex",
        "QXmlQuery",
        "QXmlResultItems",
        "QXmlSchema",
        "QXmlSchemaValidator",
        "QXmlSerializer",
    ],
}



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from PySide6.QtCore import QObject, QThread, Signal, Slot
from PySide6.QtWidgets import QApplication, QPushButton, QVBoxLayout, QWidget


# Create a basic window with a layout and a button
class MainForm(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("My Form")
        self.layout = QVBoxLayout()
        self.button = QPushButton("Click me!")
        self.button.clicked.connect(self.start_thread)
        self.layout.addWidget(self.button)
        self.setLayout(self.layout)

    # Instantiate and start a new thread
    def start_thread(self):
        instanced_thread = WorkerThread(self)
        instanced_thread.start()

    # Create the Slots that will receive signals
    @Slot(str)
    def update_str_field(self, message):
        print(message)

    @Slot(int)
    def update_int_field(self, value):
        print(value)


# Signals must inherit QObject
class MySignals(QObject):
    signal_str = Signal(str)
    signal_int = Signal(int)


# Create the Worker Thread
class WorkerThread(QThread):
    def __init__(self, parent=None):
        QThread.__init__(self, parent)
        # Instantiate signals and connect signals to the slots
        self.signals = MySignals()
        self.signals.signal_str.connect(parent.update_str_field)
        self.signals.signal_int.connect(parent.update_int_field)

    def run(self):
        # Do something on the worker thread
        a = 1 + 1
        # Emit signals whenever you want
        self.signals.signal_int.emit(a)
        self.signals.signal_str.emit("This text comes to Main thread from our Worker thread.")


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MainForm()
    window.show()
    sys.exit(app.exec())



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QUrlQuery'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QUrlQuery


class QUrlQueryTest(unittest.TestCase):
    def testConstructing(self):
        empty = QUrlQuery()
        self.assertTrue(empty.isEmpty())

        empty.clear()
        self.assertTrue(empty.isEmpty())

    def testAddRemove(self):
        query = QUrlQuery()

        query.addQueryItem("a", "b")
        self.assertTrue(not query.isEmpty())
        self.assertTrue(query.hasQueryItem("a"))
        self.assertEqual(query.queryItemValue("a"), "b")
        self.assertEqual(query.allQueryItemValues("a"), ["b"])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest
import warnings

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectType


class TestDeprecatedCall(unittest.TestCase):
    def testCallWithError(self):
        o = ObjectType()
        warnings.simplefilter('error')
        self.assertRaises(DeprecationWarning, o.deprecatedFunction)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QLocale'''

import ctypes
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QLocale


class QLocaleTestToNumber(unittest.TestCase):
    def testToNumberInt(self):
        obj = QLocale(QLocale.Language.C)
        self.assertEqual((37, True), obj.toInt('37'))

    def testToNumberFloat(self):
        obj = QLocale(QLocale.Language.C)
        self.assertEqual((ctypes.c_float(37.109).value, True),
                         obj.toFloat('37.109'))

    def testToNumberDouble(self):
        obj = QLocale(QLocale.Language.C)
        self.assertEqual((ctypes.c_double(37.109).value, True),
                         obj.toDouble('37.109'))

    def testToNumberShort(self):
        obj = QLocale(QLocale.Language.C)
        self.assertEqual((ctypes.c_short(37).value, True),
                         obj.toShort('37'))

    def testToNumberLong(self):
        obj = QLocale(QLocale.Language.C)
        self.assertEqual((ctypes.c_long(37).value, True),
                         obj.toLong('37'))

    def testToNumberULongLong(self):
        obj = QLocale(QLocale.Language.C)
        self.assertEqual((ctypes.c_ulonglong(37).value, True),
                         obj.toULongLong('37'))

    def testToNumberULongLongNegative(self):
        obj = QLocale(QLocale.Language.C)
        self.assertTrue(not obj.toULongLong('-37')[1])

    def testToCurrencyString(self):
        """PYSIDE-2133, do not use int overload, dropping decimals."""
        en_locale = QLocale("en_US")
        value = en_locale.toCurrencyString(1234.56)
        self.assertEqual(value, "$1,234.56")

    def testToString(self):
        """PYSIDE-2168, check negative values"""
        en_locale = QLocale("en_US")
        value = en_locale.toString(-4)
        self.assertEqual(value, "-4")
        # Verify that large types (long long/double) are used.
        value = en_locale.toString(3000000000)
        self.assertEqual(value, "3,000,000,000")
        value = en_locale.toString(10e40)
        self.assertEqual(value.lower(), "1e+41")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' unit test for BUG #1060 '''

import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication
from PySide6.QtUiTools import QUiLoader


class MyQUiLoader(QUiLoader):
    def __init__(self):
        super().__init__()

    def createWidget(self, *args):
        return super(MyQUiLoader, self).createWidget(*args)


if __name__ == "__main__":
    app = QApplication([])

    file = Path(__file__).resolve().parent / 'bug_1060.ui'
    assert (file.is_file())
    ui = MyQUiLoader().load(file)
    ui.show()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit test for WebView'''

import os
import sys
import unittest
from pathlib import Path

# Append the necessary paths to sys.path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWebView import QtWebView

from helper.usesqapplication import UsesQApplication


class QWebViewTestCase(UsesQApplication):
    def test_webview_exists(self):
        # Initialize QtWebView
        QtWebView.initialize()

        # Check if QtWebView can be initialized
        self.assertTrue(QtWebView is not None)
        print("QtWebView is available in PySide6.")


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''(Very) Simple test case for missing names from QtGui and QtWidgets'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6 import QtGui
from PySide6 import QtWidgets


class MissingClasses(unittest.TestCase):
    def testQDrag(self):  # Bug 222
        getattr(QtGui, 'QDrag')

    def testQDropEvent(self):  # Bug 255
        getattr(QtGui, 'QDropEvent')


class MissingMembers(unittest.TestCase):

    def testQFontMetricsSize(self):  # Bug 223
        QtGui.QFontMetrics.size

    def testQLayoutSetSpacing(self):  # Bug 231
        QtWidgets.QLayout.setSpacing

    def testQImageLoad(self):  # Bug 257
        QtGui.QImage.load

    def testQStandardItemModelinsertRow(self):  # Bug 227
        QtGui.QStandardItemModel.insertRow


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

# PySide6 tutorial 14


import sys
import math
import random

from PySide6.QtCore import (QPoint, QRect, QTime, QTimer, QSize, Qt,
                            Signal, Slot, qWarning)
from PySide6.QtGui import (QColor, QFont, QKeySequence, QPainter, QPainterStateGuard,
                           QPalette, QShortcut, QRegion, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QGridLayout, QHBoxLayout,
                               QLabel, QLCDNumber, QPushButton, QSizePolicy,
                               QSlider, QVBoxLayout, QWidget)


class LCDRange(QWidget):

    value_changed = Signal(int)

    def __init__(self, text=None, parent=None):
        if isinstance(text, QWidget):
            parent = text
            text = None

        super().__init__(parent)

        self.init()

        if text:
            self.set_text(text)

    def init(self):
        lcd = QLCDNumber(2)
        self.slider = QSlider(Qt.Orientation.Horizontal)
        self.slider.setRange(0, 99)
        self.slider.setValue(0)
        self.label = QLabel()
        self.label.setAlignment(Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignTop)
        self.label.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)

        self.slider.valueChanged.connect(lcd.display)
        self.slider.valueChanged.connect(self.value_changed)

        layout = QVBoxLayout(self)
        layout.addWidget(lcd)
        layout.addWidget(self.slider)
        layout.addWidget(self.label)

        self.setFocusProxy(self.slider)

    def value(self):
        return self.slider.value()

    @Slot(int)
    def set_value(self, value):
        self.slider.setValue(value)

    def text(self):
        return self.label.text()

    def set_range(self, minValue, maxValue):
        if minValue < 0 or maxValue > 99 or minValue > maxValue:
            qWarning(f"LCDRange::setRange({minValue}, {maxValue})\n"
                     "\tRange must be 0..99\n"
                     "\tand minValue must not be greater than maxValue")
            return

        self.slider.setRange(minValue, maxValue)

    def set_text(self, text):
        self.label.setText(text)


class CannonField(QWidget):

    angle_changed = Signal(int)
    force_changed = Signal(int)
    hit = Signal()
    missed = Signal()
    can_shoot = Signal(bool)

    def __init__(self, parent=None):
        super().__init__(parent)

        self._current_angle = 45
        self._current_force = 0
        self._timer_count = 0
        self._auto_shoot_timer = QTimer(self)
        self._auto_shoot_timer.timeout.connect(self.move_shot)
        self._shoot_angle = 0
        self._shoot_force = 0
        self.target = QPoint(0, 0)
        self._game_ended = False
        self._barrel_pressed = False
        self.setPalette(QPalette(QColor(250, 250, 200)))
        self.setAutoFillBackground(True)
        self.new_target()

    def angle(self):
        return self._current_angle

    @Slot(int)
    def set_angle(self, angle):
        if angle < 5:
            angle = 5
        if angle > 70:
            angle = 70
        if self._current_angle == angle:
            return
        self._current_angle = angle
        self.update()
        self.angle_changed.emit(self._current_angle)

    def force(self):
        return self._current_force

    @Slot(int)
    def set_force(self, force):
        if force < 0:
            force = 0
        if self._current_force == force:
            return
        self._current_force = force
        self.force_changed.emit(self._current_force)

    @Slot()
    def shoot(self):
        if self.is_shooting():
            return
        self._timer_count = 0
        self._shoot_angle = self._current_angle
        self._shoot_force = self._current_force
        self._auto_shoot_timer.start(5)
        self.can_shoot.emit(False)

    first_time = True

    def new_target(self):
        if CannonField.first_time:
            CannonField.first_time = False
            midnight = QTime(0, 0, 0)
            random.seed(midnight.secsTo(QTime.currentTime()))

        self.target = QPoint(200 + random.randint(0, 190 - 1), 10 + random.randint(0, 255 - 1))
        self.update()

    def set_game_over(self):
        if self._game_ended:
            return
        if self.is_shooting():
            self._auto_shoot_timer.stop()
        self._game_ended = True
        self.update()

    def restart_game(self):
        if self.is_shooting():
            self._auto_shoot_timer.stop()
        self._game_ended = False
        self.update()
        self.can_shoot.emit(True)

    @Slot()
    def move_shot(self):
        region = QRegion(self.shot_rect())
        self._timer_count += 1

        shot_r = self.shot_rect()

        if shot_r.intersects(self.target_rect()):
            self._auto_shoot_timer.stop()
            self.hit.emit()
            self.can_shoot.emit(True)
        elif (shot_r.x() > self.width() or shot_r.y() > self.height()
                or shot_r.intersects(self.barrier_rect())):
            self._auto_shoot_timer.stop()
            self.missed.emit()
            self.can_shoot.emit(True)
        else:
            region = region.united(QRegion(shot_r))

        self.update(region)

    def mousePressEvent(self, event):
        if event.button() != Qt.MouseButton.LeftButton:
            return
        if self.barrel_hit(event.position().toPoint()):
            self._barrel_pressed = True

    def mouseMoveEvent(self, event):
        if not self._barrel_pressed:
            return
        pos = event.position().toPoint()
        if pos.x() <= 0:
            pos.setX(1)
        if pos.y() >= self.height():
            pos.setY(self.height() - 1)
        rad = math.atan((float(self.rect().bottom()) - pos.y()) / pos.x())
        self.set_angle(round(rad * 180 / math.pi))

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.MouseButton.LeftButton:
            self._barrel_pressed = False

    def paintEvent(self, event):
        with QPainter(self) as painter:
            if self._game_ended:
                painter.setPen(Qt.GlobalColor.black)
                painter.setFont(QFont("Courier", 48, QFont.Weight.Bold))
                painter.drawText(self.rect(), Qt.AlignmentFlag.AlignCenter, "Game Over")

            self.paint_cannon(painter)
            self.paint_barrier(painter)
            if self.is_shooting():
                self.paint_shot(painter)
            if not self._game_ended:
                self.paint_target(painter)

    def paint_shot(self, painter):
        painter.setPen(Qt.PenStyle.NoPen)
        painter.setBrush(Qt.GlobalColor.black)
        painter.drawRect(self.shot_rect())

    def paint_target(self, painter):
        painter.setPen(Qt.GlobalColor.black)
        painter.setBrush(Qt.GlobalColor.red)
        painter.drawRect(self.target_rect())

    def paint_barrier(self, painter):
        painter.setPen(Qt.GlobalColor.black)
        painter.setBrush(Qt.GlobalColor.yellow)
        painter.drawRect(self.barrier_rect())

    barrel_rect = QRect(33, -4, 15, 8)

    def paint_cannon(self, painter):
        painter.setPen(Qt.PenStyle.NoPen)
        painter.setBrush(Qt.GlobalColor.blue)

        with QPainterStateGuard(painter):
            painter.translate(0, self.height())
            painter.drawPie(QRect(-35, -35, 70, 70), 0, 90 * 16)
            painter.rotate(-self._current_angle)
            painter.drawRect(CannonField.barrel_rect)

    def cannon_rect(self):
        result = QRect(0, 0, 50, 50)
        result.moveBottomLeft(self.rect().bottomLect())
        return result

    def shot_rect(self):
        gravity = 4.0

        time = self._timer_count / 40.0
        velocity = self._shoot_force
        radians = self._shoot_angle * math.pi / 180

        velx = velocity * math.cos(radians)
        vely = velocity * math.sin(radians)
        x0 = (CannonField.barrel_rect.right() + 5) * math.cos(radians)
        y0 = (CannonField.barrel_rect.right() + 5) * math.sin(radians)
        x = x0 + velx * time
        y = y0 + vely * time - 0.5 * gravity * time * time

        result = QRect(0, 0, 6, 6)
        result.moveCenter(QPoint(round(x), self.height() - 1 - round(y)))
        return result

    def target_rect(self):
        result = QRect(0, 0, 20, 10)
        result.moveCenter(QPoint(self.target.x(), self.height() - 1 - self.target.y()))
        return result

    def barrier_rect(self):
        return QRect(145, self.height() - 100, 15, 99)

    def barrel_hit(self, pos):
        matrix = QTransform()
        matrix.translate(0, self.height())
        matrix.rotate(-self._current_angle)
        matrix, invertible = matrix.inverted()
        return self.barrel_rect.contains(matrix.map(pos))

    def game_over(self):
        return self._game_ended

    def is_shooting(self):
        return self._auto_shoot_timer.isActive()

    def sizeHint(self):
        return QSize(400, 300)


class GameBoard(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        quit = QPushButton("&Quit")
        quit.setFont(QFont("Times", 18, QFont.Weight.Bold))

        quit.clicked.connect(qApp.quit)  # noqa: F821

        angle = LCDRange("ANGLE")
        angle.set_range(5, 70)

        force = LCDRange("FORCE")
        force.set_range(10, 50)

        cannon_box = QFrame()
        cannon_box.setFrameStyle(QFrame.Shape.WinPanel | QFrame.Shadow.Sunken)

        self._cannon_field = CannonField()

        angle.value_changed.connect(self._cannon_field.set_angle)
        self._cannon_field.angle_changed.connect(angle.set_value)

        force.value_changed.connect(self._cannon_field.set_force)
        self._cannon_field.force_changed.connect(force.set_value)

        self._cannon_field.hit.connect(self.hit)
        self._cannon_field.missed.connect(self.missed)

        shoot = QPushButton("&Shoot")
        shoot.setFont(QFont("Times", 18, QFont.Weight.Bold))

        shoot.clicked.connect(self.fire)
        self._cannon_field.can_shoot.connect(shoot.setEnabled)

        restart = QPushButton("&New Game")
        restart.setFont(QFont("Times", 18, QFont.Weight.Bold))

        restart.clicked.connect(self.new_game)

        self.hits = QLCDNumber(2)
        self._shots_left = QLCDNumber(2)
        hits_label = QLabel("HITS")
        shots_left_label = QLabel("SHOTS LEFT")

        QShortcut(QKeySequence(Qt.Key.Key_Enter), self, self.fire)
        QShortcut(QKeySequence(Qt.Key.Key_Return), self, self.fire)
        QShortcut(QKeySequence(Qt.Modifier.CTRL | Qt.Key.Key_Q), self, self.close)

        top_layout = QHBoxLayout()
        top_layout.addWidget(shoot)
        top_layout.addWidget(self.hits)
        top_layout.addWidget(hits_label)
        top_layout.addWidget(self._shots_left)
        top_layout.addWidget(shots_left_label)
        top_layout.addStretch(1)
        top_layout.addWidget(restart)

        left_layout = QVBoxLayout()
        left_layout.addWidget(angle)
        left_layout.addWidget(force)

        cannon_layout = QVBoxLayout()
        cannon_layout.addWidget(self._cannon_field)
        cannon_box.setLayout(cannon_layout)

        grid_layout = QGridLayout(self)
        grid_layout.addWidget(quit, 0, 0)
        grid_layout.addLayout(top_layout, 0, 1)
        grid_layout.addLayout(left_layout, 1, 0)
        grid_layout.addWidget(cannon_box, 1, 1, 2, 1)
        grid_layout.setColumnStretch(1, 10)

        angle.set_value(60)
        force.set_value(25)
        angle.setFocus()

        self.new_game()

    @Slot()
    def fire(self):
        if self._cannon_field.game_over() or self._cannon_field.is_shooting():
            return
        self._shots_left.display(self._shots_left.intValue() - 1)
        self._cannon_field.shoot()

    @Slot()
    def hit(self):
        self.hits.display(self.hits.intValue() + 1)
        if self._shots_left.intValue() == 0:
            self._cannon_field.set_game_over()
        else:
            self._cannon_field.new_target()

    @Slot()
    def missed(self):
        if self._shots_left.intValue() == 0:
            self._cannon_field.set_game_over()

    @Slot()
    def new_game(self):
        self._shots_left.display(15)
        self.hits.display(0)
        self._cannon_field.restart_game()
        self._cannon_field.new_target()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    board = GameBoard()
    board.setGeometry(100, 100, 500, 355)
    board.show()
    sys.exit(app.exec())



# Copyright (C) 2026 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

import sys
from pathlib import Path

from PySide6.QtCore import QCoreApplication, QSize, Qt
from PySide6.QtGui import QGuiApplication, QShortcut, QKeySequence
from PySide6.QtQuick import QQuickView

from graphprinter import GraphPrinter


if __name__ == "__main__":
    app = QGuiApplication(sys.argv)

    viewer = QQuickView()
    viewer.setTitle("Graph Printing")
    graphPrinter = GraphPrinter()
    viewer.rootContext().setContextProperty("graphPrinter", graphPrinter)
    viewer.setMinimumSize(QSize(1280, 720))
    viewer.engine().addImportPath(Path(__file__).parent)
    viewer.loadFromModule("GraphPrintingExample", "Main")
    window = viewer.rootObject()
    if not window:
        sys.exit(-1)
    quitKey = QKeySequence(QKeySequence.StandardKey.Quit)
    if not quitKey.isEmpty():
        quitShortcut = QShortcut(quitKey, window)
        quitShortcut.activated.connect(app.quit)
        quitShortcut.setContext(Qt.ShortcutContext.ApplicationShortcut)
    viewer.setResizeMode(QQuickView.ResizeMode.SizeRootObjectToView)
    viewer.setColor(Qt.GlobalColor.white)
    viewer.show()

    ex = QCoreApplication.exec()
    del viewer
    sys.exit(ex)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QFile, QObject


class TestBugPYSIDE42 (unittest.TestCase):

    def testIt(self):
        obj1 = QFile()
        obj2 = QObject()
        hash1 = hash(obj1)
        hash2 = hash(obj2)

        self.assertNotEqual(hash1, hash2)
        self.assertEqual(hash1, hash(obj1))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the QML Polar Chart Example from Qt v5.x"""

import os
from pathlib import Path
import sys

from PySide6.QtQuick import QQuickView
from PySide6.QtCore import QUrl
from PySide6.QtWidgets import QApplication


if __name__ == '__main__':
    app = QApplication(sys.argv)
    viewer = QQuickView()

    src_dir = Path(__file__).resolve().parent
    viewer.engine().addImportPath(os.fspath(src_dir))
    viewer.engine().quit.connect(viewer.close)

    viewer.setTitle = "QML Polar Chart"
    viewer.setSource(QUrl.fromLocalFile(src_dir / 'main.qml'))
    viewer.setResizeMode(QQuickView.SizeRootObjectToView)
    viewer.show()

    sys.exit(app.exec())
